#!/usr/bin/env python
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Shows how to add product partitions to a Performance Max retail campaign.

For Performance Max campaigns, product partitions are represented using the
AssetGroupListingGroupFilter resource. This resource can be combined with
itself to form a hierarchy that creates a product partition tree.

For more information about Performance Max retail campaigns, see the
shopping_ads/add_performance_max_retail_campaign.py example.
"""


import argparse
import sys
from typing import Dict, List, Optional

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.resources.types.asset_group_listing_group_filter import (
    ListingGroupFilterDimension,
)
from google.ads.googleads.v22.resources.types.asset_group_listing_group_filter import (
    AssetGroupListingGroupFilter,
)
from google.ads.googleads.v22.services.services.google_ads_service import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.types.google_ads_service import (
    MutateGoogleAdsResponse,
    SearchGoogleAdsRequest,
    SearchGoogleAdsResponse,
)
from google.ads.googleads.v22.services.types.google_ads_service import (
    MutateOperation,
)


# We specify temporary IDs that are specific to a single mutate request.
# Temporary IDs are always negative and unique within one mutate request.
#
# See https://developers.google.com/google-ads/api/docs/mutating/best-practices
# for further details.
#
# These temporary IDs are fixed because they are used in multiple places.
_TEMPORARY_ID_LISTING_GROUP_ROOT: int = -1


class AssetGroupListingGroupFilterRemoveOperationFactory:
    def __init__(
        self,
        client: GoogleAdsClient,
        listing_group_filters: List[AssetGroupListingGroupFilter],
    ):
        """Factory class for creating sorted list of MutateOperations.

        The operations remove the given tree of AssetGroupListingGroupFilters,
        When removing these listing group filters, the remove operations must be
        sent in a specific order that removes leaf nodes before removing parent
        nodes.

        Args:
            client: an initialized GoogleAdsClient instance.
            listing_group_filters: a list of AssetGroupListingGroupFilters.
        """
        if not listing_group_filters:
            raise ValueError("No listing group filters to remove.")

        self.client: GoogleAdsClient = client
        self.root_resource_name: Optional[str] = None
        self.parents_to_children: Dict[str, List[str]] = {}

        # Process the given list of listing group filters to identify the root
        # node and any parent to child edges in the tree.
        for listing_group_filter_node in listing_group_filters:
            resource_name: str = listing_group_filter_node.resource_name
            parent_resource_name: Optional[str] = (
                listing_group_filter_node.parent_listing_group_filter
            )

            # When the node has no parent, it means it's the root node.
            if not parent_resource_name:
                if self.root_resource_name:
                    # Check if another root node has already been detected and
                    # raise an error if so, as only one root node can exist for
                    # a given tree.
                    raise ValueError("More than one listing group parent node.")
                else:
                    self.root_resource_name = resource_name
            else:
                # Check if we've already visited a sibling in this group, and
                # either update it or create a new branch accordingly.
                if parent_resource_name in self.parents_to_children:
                    # If we've visited a sibling already, add this resource
                    # name to the existing list.
                    self.parents_to_children[parent_resource_name].append(
                        resource_name
                    )
                else:
                    # If we haven't visited any siblings, then create a new list
                    # for this parent node and add this resource name to it.
                    self.parents_to_children[parent_resource_name] = [
                        resource_name
                    ]

    # [START add_performance_max_product_listing_group_tree_2]
    def remove_all(self) -> List[MutateOperation]:
        """Creates a list of MutateOperations for the listing group filter tree.

        Returns:
            A list of MutateOperations that remove each specified
            AssetGroupListingGroupFilter in the tree passed in when this
            class was initialized.
        """
        if not self.root_resource_name:
            # This case should ideally be prevented by the __init__ check,
            # but as a safeguard for type checking remove_descendants_and_filter.
            return []
        return self.remove_descendants_and_filter(self.root_resource_name)
        # [END add_performance_max_product_listing_group_tree_2]

    # [START add_performance_max_product_listing_group_tree_3]
    def remove_descendants_and_filter(
        self, resource_name: str
    ) -> List[MutateOperation]:
        """Builds a post-order sorted list of MutateOperations.

        Creates a list of MutateOperations that remove all the descendents of
        the specified AssetGroupListingGroupFilter resource name. The order of
        removal is post-order, where all the children (and their children,
        recursively) are removed first. Then, the root node itself is removed.

        Args:
            resource_name: an AssetGroupListingGroupFilter resource name.

        Returns:
            a sorted list of MutateOperations.
        """
        operations: List[MutateOperation] = []

        # Check if resource name is a parent.
        if resource_name in self.parents_to_children:
            # If this resource name is a parent, call this method recursively
            # on each of its children.
            for child_resource_name in self.parents_to_children[resource_name]:
                operations.extend(
                    self.remove_descendants_and_filter(child_resource_name)
                )

        mutate_operation: MutateOperation = self.client.get_type(
            "MutateOperation"
        )
        mutate_operation.asset_group_listing_group_filter_operation.remove = (
            resource_name
        )
        operations.append(mutate_operation)

        return operations
        # [END add_performance_max_product_listing_group_tree_3]


class AssetGroupListingGroupFilterCreateOperationFactory:
    def __init__(
        self,
        client: GoogleAdsClient,
        customer_id: str,
        asset_group_id: int,  # Will be str for path construction
        root_listing_id: int,
    ):
        """A factory class for creating MutateOperations.

        These operations create new AssetGroupListingGroupFilterMutateOperation
        instances using the given customer ID and asset group ID.

        Args:
            client: an initialized GoogleAdsClient instance.
            customer_id: a client customer ID.
            asset_group_id: the asset group id for the Performance Max campaign.
            root_listing_id: a temporary ID to use as the listing group root.
        """
        self.client: GoogleAdsClient = client
        self.customer_id: str = customer_id
        # asset_group_id is used as a string in path construction.
        self.asset_group_id: str = str(asset_group_id)
        self.root_listing_id: int = root_listing_id
        self.next_temp_id: int = self.root_listing_id - 1

    def next_id(self) -> int:
        """Returns the next temporary ID for use in a sequence.

        The temporary IDs are used in the list of MutateOperations in order to
        refer to objects in the request that aren't in the API yet. For more
        details see:
        https://developers.google.com/google-ads/api/docs/mutating/best-practices#temporary_resource_names

        Returns:
            A new temporary ID.
        """
        self.next_temp_id -= 1
        return self.next_temp_id

    # [START add_performance_max_product_listing_group_tree_4]
    def create_root(self) -> MutateOperation:
        """Creates a MutateOperation to add a root AssetGroupListingGroupFilter.

        Returns:
            A MutateOperation for a new AssetGroupListingGroupFilter.
        """
        googleads_service: GoogleAdsServiceClient = self.client.get_service(
            "GoogleAdsService"
        )

        mutate_operation: MutateOperation = self.client.get_type(
            "MutateOperation"
        )
        asset_group_listing_group_filter: AssetGroupListingGroupFilter = (
            mutate_operation.asset_group_listing_group_filter_operation.create
        )

        asset_group_listing_group_filter.resource_name = (
            googleads_service.asset_group_listing_group_filter_path(
                self.customer_id,
                self.asset_group_id,
                str(self.root_listing_id),
            )
        )
        asset_group_listing_group_filter.asset_group = (
            googleads_service.asset_group_path(
                self.customer_id, self.asset_group_id
            )
        )
        # Since this is the root node, do not set the
        # parent_listing_group_filter field. For all other nodes, this would
        # refer to the parent listing group filter resource name.
        # asset_group_listing_group_filter.parent_listing_group_filter = "<PARENT FILTER NAME>"

        # Unlike the add_performance_max_retail_campaign example, the type for
        # the root node here must be a subdivision because we add child
        # partitions under it.
        asset_group_listing_group_filter.type_ = (
            self.client.enums.ListingGroupFilterTypeEnum.SUBDIVISION
        )

        # Because this is a Performance Max campaign for retail, we need to
        # specify that this is a shopping listing source.
        asset_group_listing_group_filter.listing_source = (
            self.client.enums.ListingGroupFilterListingSourceEnum.SHOPPING
        )

        return mutate_operation
        # [END add_performance_max_product_listing_group_tree_4]

    # [START add_performance_max_product_listing_group_tree_5]
    def create_subdivision(
        self,
        parent_id: int,
        temporary_id: int,
        dimension: ListingGroupFilterDimension,
    ) -> MutateOperation:
        """Creates a MutateOperation to add an AssetGroupListingGroupFilter.

        Use this method if the filter will have child filters. Otherwise use
        the create_unit method.

        Args:
            parent_id: the ID of the parent AssetGroupListingGroupFilter.
            temporary_id: a temporary ID for the operation being created.
            dimension: The dimension to associate with this new
                AssetGroupListingGroupFilter.

        Returns:
            a MutateOperation for a new AssetGroupListingGroupFilter
        """
        googleads_service: GoogleAdsServiceClient = self.client.get_service(
            "GoogleAdsService"
        )

        mutate_operation: MutateOperation = self.client.get_type(
            "MutateOperation"
        )
        asset_group_listing_group_filter: AssetGroupListingGroupFilter = (
            mutate_operation.asset_group_listing_group_filter_operation.create
        )

        asset_group_listing_group_filter.resource_name = (
            googleads_service.asset_group_listing_group_filter_path(
                self.customer_id, self.asset_group_id, str(temporary_id)
            )
        )
        asset_group_listing_group_filter.asset_group = (
            googleads_service.asset_group_path(
                self.customer_id, self.asset_group_id
            )
        )
        asset_group_listing_group_filter.parent_listing_group_filter = (
            googleads_service.asset_group_listing_group_filter_path(
                self.customer_id, self.asset_group_id, str(parent_id)
            )
        )
        # We must use the Subdivision type to indicate that the
        # AssetGroupListingGroupFilter will have children.
        asset_group_listing_group_filter.type_ = (
            self.client.enums.ListingGroupFilterTypeEnum.SUBDIVISION
        )
        # Because this is a Performance Max campaign for retail, we need to
        # specify that this is in the shopping listing source.
        asset_group_listing_group_filter.listing_source = (
            self.client.enums.ListingGroupFilterListingSourceEnum.SHOPPING
        )
        asset_group_listing_group_filter.case_value = dimension

        return mutate_operation
        # [END add_performance_max_product_listing_group_tree_5]

    # [START add_performance_max_product_listing_group_tree_6]
    def create_unit(
        self,
        parent_id: int,
        temporary_id: int,
        dimension: ListingGroupFilterDimension,
    ) -> MutateOperation:
        """Creates a MutateOperation to add an AssetGroupListingGroupFilter.

        Use this method if the filter will not have child filters. Otherwise use
        the create_subdivision method.

        Args:
            parent_id: the ID of the parent AssetGroupListingGroupFilter.
            temporary_id: a temporary ID for the operation being created.
            dimension: The dimension to associate with this new
                AssetGroupListingGroupFilter.

        Returns:
            a MutateOperation for a new AssetGroupListingGroupFilter
        """
        googleads_service: GoogleAdsServiceClient = self.client.get_service(
            "GoogleAdsService"
        )

        mutate_operation: MutateOperation = self.client.get_type(
            "MutateOperation"
        )
        asset_group_listing_group_filter: AssetGroupListingGroupFilter = (
            mutate_operation.asset_group_listing_group_filter_operation.create
        )

        asset_group_listing_group_filter.resource_name = (
            googleads_service.asset_group_listing_group_filter_path(
                self.customer_id, self.asset_group_id, str(temporary_id)
            )
        )
        asset_group_listing_group_filter.asset_group = (
            googleads_service.asset_group_path(
                self.customer_id, self.asset_group_id
            )
        )
        asset_group_listing_group_filter.parent_listing_group_filter = (
            googleads_service.asset_group_listing_group_filter_path(
                self.customer_id, self.asset_group_id, str(parent_id)
            )
        )
        # We must use the UnitIncluded type to indicate that the
        # AssetGroupListingGroupFilter won't have children.
        asset_group_listing_group_filter.type_ = (
            self.client.enums.ListingGroupFilterTypeEnum.UNIT_INCLUDED
        )
        # Because this is a Performance Max campaign for retail, we need to
        # specify that this is in the shopping listing source.
        asset_group_listing_group_filter.listing_source = (
            self.client.enums.ListingGroupFilterListingSourceEnum.SHOPPING
        )
        asset_group_listing_group_filter.case_value = dimension

        return mutate_operation
        # [END add_performance_max_product_listing_group_tree_6]


# [START add_performance_max_product_listing_group_tree]
def main(
    client: GoogleAdsClient,
    customer_id: str,
    asset_group_id: int,  # Will be str for path construction
    replace_existing_tree: bool,
) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        asset_group_id: the asset group id for the Performance Max campaign.
        replace_existing_tree: option to remove existing product tree from the
            passed in asset group.
    """
    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )
    # asset_group_id is used as a string in path construction.
    asset_group_resource_name: str = googleads_service.asset_group_path(
        customer_id, str(asset_group_id)
    )
    operations: List[MutateOperation] = []

    if replace_existing_tree:
        # Retrieve a list of existing AssetGroupListingGroupFilters
        existing_listing_group_filters: List[AssetGroupListingGroupFilter] = (
            get_all_existing_listing_group_filter_assets_in_asset_group(
                client, customer_id, asset_group_resource_name
            )
        )

        # If present, create MutateOperations to remove each
        # AssetGroupListingGroupFilter and add them to the list of operations.
        if existing_listing_group_filters:
            remove_operation_factory = (
                AssetGroupListingGroupFilterRemoveOperationFactory(
                    client, existing_listing_group_filters
                )
            )
            operations.extend(remove_operation_factory.remove_all())

    create_operation_factory = (
        AssetGroupListingGroupFilterCreateOperationFactory(
            client,
            customer_id,
            asset_group_id,  # Pass as int, will be converted to str in __init__
            _TEMPORARY_ID_LISTING_GROUP_ROOT,
        )
    )

    operations.append(create_operation_factory.create_root())

    new_dimension: ListingGroupFilterDimension = client.get_type(
        "ListingGroupFilterDimension"
    )
    new_dimension.product_condition.condition = (
        client.enums.ListingGroupFilterProductConditionEnum.NEW
    )
    operations.append(
        create_operation_factory.create_unit(
            _TEMPORARY_ID_LISTING_GROUP_ROOT,
            create_operation_factory.next_id(),
            new_dimension,
        )
    )

    used_dimension: ListingGroupFilterDimension = client.get_type(
        "ListingGroupFilterDimension"
    )
    used_dimension.product_condition.condition = (
        client.enums.ListingGroupFilterProductConditionEnum.USED
    )
    operations.append(
        create_operation_factory.create_unit(
            _TEMPORARY_ID_LISTING_GROUP_ROOT,
            create_operation_factory.next_id(),
            used_dimension,
        )
    )

    # We save this ID because create child nodes underneath it.
    subdivision_id_condition_other: int = create_operation_factory.next_id()

    # All sibling nodes must have the same dimension type. We use an empty
    # product_condition to indicate that this is an "Other" partition.
    other_dimension: ListingGroupFilterDimension = client.get_type(
        "ListingGroupFilterDimension"
    )
    # This triggers the presence of the product_condition field without
    # specifying any field values. This is important in order to tell the API
    # that this is an "other" node.
    other_dimension.product_condition._pb.SetInParent()
    # We're calling create_subdivision because this listing group will have
    # children.
    operations.append(
        create_operation_factory.create_subdivision(
            _TEMPORARY_ID_LISTING_GROUP_ROOT,
            subdivision_id_condition_other,
            other_dimension,
        )
    )

    cool_dimension: ListingGroupFilterDimension = client.get_type(
        "ListingGroupFilterDimension"
    )
    cool_dimension.product_brand.value = "CoolBrand"
    operations.append(
        create_operation_factory.create_unit(
            subdivision_id_condition_other,
            create_operation_factory.next_id(),
            cool_dimension,
        )
    )

    cheap_dimension: ListingGroupFilterDimension = client.get_type(
        "ListingGroupFilterDimension"
    )
    cheap_dimension.product_brand.value = "CheapBrand"
    operations.append(
        create_operation_factory.create_unit(
            subdivision_id_condition_other,
            create_operation_factory.next_id(),
            cheap_dimension,
        )
    )

    empty_dimension: ListingGroupFilterDimension = client.get_type(
        "ListingGroupFilterDimension"
    )
    # This triggers the presence of the product_brand field without specifying
    # any field values. This is important in order to tell the API
    # that this is an "other" node.
    empty_dimension.product_brand._pb.SetInParent()
    operations.append(
        create_operation_factory.create_unit(
            subdivision_id_condition_other,
            create_operation_factory.next_id(),
            empty_dimension,
        )
    )

    response: MutateGoogleAdsResponse = googleads_service.mutate(
        customer_id=customer_id, mutate_operations=operations
    )

    print_response_details(operations, response)
    # [END add_performance_max_product_listing_group_tree]


# [START add_performance_max_product_listing_group_tree_7]
def get_all_existing_listing_group_filter_assets_in_asset_group(
    client: GoogleAdsClient,
    customer_id: str,
    asset_group_resource_name: str,
) -> List[AssetGroupListingGroupFilter]:
    """Fetches all of the listing group filters in an asset group.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        asset_group_resource_name: the asset group resource name for the
            Performance Max campaign.

    Returns:
        a list of AssetGroupListingGroupFilters.
    """
    query: str = f"""
        SELECT
          asset_group_listing_group_filter.resource_name,
          asset_group_listing_group_filter.parent_listing_group_filter
        FROM asset_group_listing_group_filter
        WHERE asset_group_listing_group_filter.asset_group = '{asset_group_resource_name}'"""

    request: SearchGoogleAdsRequest = client.get_type("SearchGoogleAdsRequest")
    request.customer_id = customer_id
    request.query = query

    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )
    response: SearchGoogleAdsResponse = googleads_service.search(
        request=request
    )

    return [
        result.asset_group_listing_group_filter
        for result in response
        if result.asset_group_listing_group_filter
    ]
    # [END add_performance_max_product_listing_group_tree_7]


def print_response_details(
    mutate_operations: List[MutateOperation], response: MutateGoogleAdsResponse
) -> None:
    """Prints the details of the GoogleAdsService.Mutate request.

    This uses the original list of mutate operations to map the operation
    result to what was sent. It can be assumed that the initial set of
    operations and the list returned in the response are in the same order.

    Args:
        mutate_operations: a list of MutateOperation instances.
        response: a GoogleAdsMutateResponse instance.
    """
    # Parse the Mutate response to print details about the entities that were
    # created in the request.
    for i, result_operation in enumerate(response.mutate_operation_responses):
        requested_operation: MutateOperation = mutate_operations[i]
        resource_name: str = (
            result_operation.asset_group_listing_group_filter_result.resource_name
        )

        # Check the operation type for the requested operation in order to
        # log whether it was a remove or a create request.
        if (
            requested_operation.asset_group_listing_group_filter_operation.remove
        ):
            print(
                "Removed an AssetGroupListingGroupFilter with resource name: "
                f"'{resource_name}'."
            )
        elif (
            requested_operation.asset_group_listing_group_filter_operation.create
        ):
            print(
                "Created an AssetGroupListingGroupFilter with resource name: "
                f"'{resource_name}'."
            )
        else:
            print("An unknown operation was returned.")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description=(
            "Adds product partitions to a Performance Max retail campaign."
        )
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-a",
        "--asset_group_id",
        type=int,  # Keep as int for argparse, convert to str for API usage
        required=True,
        help="The asset group id for the Performance Max campaign.",
    )
    parser.add_argument(
        "-r",
        "--replace_existing_tree",
        action="store_true",
        help=(
            "Whether or not to replace the existing product partition tree. "
            "If the current AssetGroup already has a tree of "
            "ListingGroupFilters, attempting to add a new set of "
            "ListingGroupFilters including a root filter will result in an "
            "ASSET_GROUP_LISTING_GROUP_FILTER_ERROR_MULTIPLE_ROOTS error. "
            "Setting this option to true will remove the existing tree and "
            "prevent this error."
        ),
    )

    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(
            googleads_client,
            args.customer_id,
            args.asset_group_id,  # Pass as int, main will handle conversion
            args.replace_existing_tree,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.errors",
    marshal="google.ads.googleads.v19",
    manifest={
        "CustomInterestErrorEnum",
    },
)


class CustomInterestErrorEnum(proto.Message):
    r"""Container for enum describing possible custom interest
    errors.

    """

    class CustomInterestError(proto.Enum):
        r"""Enum describing possible custom interest errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            NAME_ALREADY_USED (2):
                Duplicate custom interest name ignoring case.
            CUSTOM_INTEREST_MEMBER_ID_AND_TYPE_PARAMETER_NOT_PRESENT_IN_REMOVE (3):
                In the remove custom interest member operation, both member
                ID and pair [type, parameter] are not present.
            TYPE_AND_PARAMETER_NOT_FOUND (4):
                The pair of [type, parameter] does not exist.
            TYPE_AND_PARAMETER_ALREADY_EXISTED (5):
                The pair of [type, parameter] already exists.
            INVALID_CUSTOM_INTEREST_MEMBER_TYPE (6):
                Unsupported custom interest member type.
            CANNOT_REMOVE_WHILE_IN_USE (7):
                Cannot remove a custom interest while it's
                still being targeted.
            CANNOT_CHANGE_TYPE (8):
                Cannot mutate custom interest type.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        NAME_ALREADY_USED = 2
        CUSTOM_INTEREST_MEMBER_ID_AND_TYPE_PARAMETER_NOT_PRESENT_IN_REMOVE = 3
        TYPE_AND_PARAMETER_NOT_FOUND = 4
        TYPE_AND_PARAMETER_ALREADY_EXISTED = 5
        INVALID_CUSTOM_INTEREST_MEMBER_TYPE = 6
        CANNOT_REMOVE_WHILE_IN_USE = 7
        CANNOT_CHANGE_TYPE = 8


__all__ = tuple(sorted(__protobuf__.manifest))



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""A set of functions to help initialize OAuth2 credentials."""


import functools
from requests import Session

from google.oauth2.service_account import Credentials as ServiceAccountCreds
from google.oauth2.credentials import Credentials as InstalledAppCredentials
from google.auth.transport.requests import Request

from google.ads.googleads import config

_SERVICE_ACCOUNT_SCOPES = ["https://www.googleapis.com/auth/adwords"]
_DEFAULT_TOKEN_URI = "https://accounts.google.com/o/oauth2/token"


def _initialize_credentials_decorator(func):
    """A decorator used to easily initialize credentials objects.

    Returns:
        An initialized credentials instance
    """

    @functools.wraps(func)
    def initialize_credentials_wrapper(*args, **kwargs):
        credentials = func(*args, **kwargs)
        # If the configs contain an http_proxy, refresh credentials through the
        # proxy URI
        proxy = kwargs.get("http_proxy")
        if proxy:
            session = Session()
            session.proxies.update({"http": proxy, "https": proxy})
            credentials.refresh(Request(session=session))
        else:
            credentials.refresh(Request())
        return credentials

    return initialize_credentials_wrapper


@_initialize_credentials_decorator
def get_installed_app_credentials(
    client_id,
    client_secret,
    refresh_token,
    http_proxy=None,
    token_uri=_DEFAULT_TOKEN_URI,
):
    """Creates and returns an instance of oauth2.credentials.Credentials.

    Args:
        client_id: A str of the oauth2 client_id from configuration.
        client_secret: A str of the oauth2 client_secret from configuration.
        refresh_token: A str of the oauth2 refresh_token from configuration.

    Returns:
        An instance of oauth2.credentials.Credentials
    """
    return InstalledAppCredentials(
        None,
        client_id=client_id,
        client_secret=client_secret,
        refresh_token=refresh_token,
        token_uri=token_uri,
    )


@_initialize_credentials_decorator
def get_service_account_credentials(
    json_key_file_path, subject, http_proxy=None, scopes=_SERVICE_ACCOUNT_SCOPES
):
    """Creates and returns an instance of oauth2.service_account.Credentials.

    Args:
        json_key_file_path: A str of the path to the private key file location.
        subject: A str of the email address of the delegated account.
        scopes: A list of additional scopes.

    Returns:
        An instance of oauth2.credentials.Credentials
    """
    return ServiceAccountCreds.from_service_account_file(
        json_key_file_path, subject=subject, scopes=scopes
    )


def get_credentials(config_data):
    """Decides which type of credentials to return based on the given config.

    Args:
        config_data: a dict containing client configuration.

    Returns:
        An initialized credentials instance.
    """
    required_installed_app_keys = config.get_oauth2_installed_app_keys()
    required_service_account_keys = (
        config.get_oauth2_required_service_account_keys()
    )

    if all(key in config_data for key in required_installed_app_keys):
        # Using the Installed App Flow
        return get_installed_app_credentials(
            config_data.get("client_id"),
            config_data.get("client_secret"),
            config_data.get("refresh_token"),
            http_proxy=config_data.get("http_proxy"),
        )
    elif all(key in config_data for key in required_service_account_keys):
        # Using the Service Account Flow
        return get_service_account_credentials(
            config_data.get("json_key_file_path"),
            config_data.get("impersonated_email"),
            http_proxy=config_data.get("http_proxy"),
        )
    else:
        raise ValueError(
            "Your YAML file is incorrectly configured for OAuth2. "
            "You need to define credentials for either the OAuth2 "
            "installed application flow ({}) or service account "
            "flow ({}).".format(
                required_installed_app_keys, required_service_account_keys
            )
        )



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "BrandRequestRejectionReasonEnum",
    },
)


class BrandRequestRejectionReasonEnum(proto.Message):
    r"""Container for enum describing rejection reasons for the
    customer brand requests.

    """

    class BrandRequestRejectionReason(proto.Enum):
        r"""Enumeration of different brand request rejection reasons.

        Values:
            UNSPECIFIED (0):
                No value has been specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            EXISTING_BRAND (2):
                Brand is already present in the commercial
                brand set.
            EXISTING_BRAND_VARIANT (3):
                Brand is already present in the commercial
                brand set, but is a variant.
            INCORRECT_INFORMATION (4):
                Brand information is not correct (eg: URL and
                name don't match).
            NOT_A_BRAND (5):
                Not a valid brand as per Google policy.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        EXISTING_BRAND = 2
        EXISTING_BRAND_VARIANT = 3
        INCORRECT_INFORMATION = 4
        NOT_A_BRAND = 5


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "AdvertisingChannelSubTypeEnum",
    },
)


class AdvertisingChannelSubTypeEnum(proto.Message):
    r"""An immutable specialization of an Advertising Channel."""

    class AdvertisingChannelSubType(proto.Enum):
        r"""Enum describing the different channel subtypes.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used as a return value only. Represents value
                unknown in this version.
            SEARCH_MOBILE_APP (2):
                Mobile app campaigns for Search.
            DISPLAY_MOBILE_APP (3):
                Mobile app campaigns for Display.
            SEARCH_EXPRESS (4):
                AdWords express campaigns for search.
            DISPLAY_EXPRESS (5):
                AdWords Express campaigns for display.
            SHOPPING_SMART_ADS (6):
                Smart Shopping campaigns.
            DISPLAY_GMAIL_AD (7):
                Gmail Ad campaigns.
            DISPLAY_SMART_CAMPAIGN (8):
                Smart display campaigns. New campaigns of
                this sub type cannot be created.
            VIDEO_ACTION (10):
                Video TrueView for Action campaigns.
            VIDEO_NON_SKIPPABLE (11):
                Video campaigns with non-skippable video ads.
            APP_CAMPAIGN (12):
                App Campaign that lets you easily promote
                your Android or iOS app across Google's top
                properties including Search, Play, YouTube, and
                the Google Display Network.
            APP_CAMPAIGN_FOR_ENGAGEMENT (13):
                App Campaign for engagement, focused on
                driving re-engagement with the app across
                several of Google's top properties including
                Search, YouTube, and the Google Display Network.
            LOCAL_CAMPAIGN (14):
                Campaigns specialized for local advertising.
            SHOPPING_COMPARISON_LISTING_ADS (15):
                Shopping Comparison Listing campaigns.
            SMART_CAMPAIGN (16):
                Standard Smart campaigns.
            VIDEO_SEQUENCE (17):
                Video campaigns with sequence video ads.
            APP_CAMPAIGN_FOR_PRE_REGISTRATION (18):
                App Campaign for pre registration,
                specialized for advertising mobile app
                pre-registration, that targets multiple
                advertising channels across Google Play, YouTube
                and Display Network. See
                https://support.google.com/google-ads/answer/9441344
                to learn more.
            VIDEO_REACH_TARGET_FREQUENCY (19):
                Video reach campaign with Target Frequency
                bidding strategy.
            TRAVEL_ACTIVITIES (20):
                Travel Activities campaigns.
            YOUTUBE_AUDIO (22):
                YouTube Audio campaigns.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        SEARCH_MOBILE_APP = 2
        DISPLAY_MOBILE_APP = 3
        SEARCH_EXPRESS = 4
        DISPLAY_EXPRESS = 5
        SHOPPING_SMART_ADS = 6
        DISPLAY_GMAIL_AD = 7
        DISPLAY_SMART_CAMPAIGN = 8
        VIDEO_ACTION = 10
        VIDEO_NON_SKIPPABLE = 11
        APP_CAMPAIGN = 12
        APP_CAMPAIGN_FOR_ENGAGEMENT = 13
        LOCAL_CAMPAIGN = 14
        SHOPPING_COMPARISON_LISTING_ADS = 15
        SMART_CAMPAIGN = 16
        VIDEO_SEQUENCE = 17
        APP_CAMPAIGN_FOR_PRE_REGISTRATION = 18
        VIDEO_REACH_TARGET_FREQUENCY = 19
        TRAVEL_ACTIVITIES = 20
        YOUTUBE_AUDIO = 22


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Searches for GoogleAdsFields that match a given prefix.

Retrieves metadata such as whether the field is selectable, filterable, or
sortable, along with the data type and the fields that are selectable with the
field. Each GoogleAdsField represents either a resource (such as customer,
campaign) or a field (such as metrics.impressions, campaign.id).
"""


import argparse
import sys

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.resources.types.google_ads_field import (
    GoogleAdsField,
)
from google.ads.googleads.v22.services.services.google_ads_field_service import (
    GoogleAdsFieldServiceClient,
)
from google.ads.googleads.v22.services.types.google_ads_field_service import (
    SearchGoogleAdsFieldsRequest,
    SearchGoogleAdsFieldsResponse,
)


def main(client: GoogleAdsClient, name_prefix: str) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        name_prefix: the name prefix to use when searching for Google Ads field
            names.
    """
    gaf_service: GoogleAdsFieldServiceClient = client.get_service(
        "GoogleAdsFieldService"
    )

    # Searches for all fields whose name begins with the specified name prefix.
    query: str = f"""
        SELECT
          name,
          category,
          selectable,
          filterable,
          sortable,
          selectable_with,
          data_type,
          is_repeated
        WHERE name LIKE '{name_prefix}%'"""

    request: SearchGoogleAdsFieldsRequest = client.get_type(
        "SearchGoogleAdsFieldsRequest"
    )
    request.query = query

    response: SearchGoogleAdsFieldsResponse = (
        gaf_service.search_google_ads_fields(request=request)
    )

    # Checks if any results were returned and exits if not.
    if response.total_results_count == 0:
        print(
            "No GoogleAdsFields found with a name that begins with "
            f"'{name_prefix}'."
        )
        sys.exit(0)

    # Retrieves each matching GoogleAdsField and prints its metadata.
    googleads_field: GoogleAdsField
    for googleads_field in response:
        print(f"{googleads_field.name}:")
        # These statements format the printed string so that the left side is
        # always a 16-character string so that the values on the right line up
        # vertically.
        print(f"{'  category:':<16}", googleads_field.category.name)
        print(f"{'  data type:':<16}", googleads_field.data_type.name)
        print(f"{'  selectable:':<16}", googleads_field.selectable)
        print(f"{'  filterable:':<16}", googleads_field.filterable)
        print(f"{'  sortable:':<16}", googleads_field.sortable)
        print(f"{'  repeated:':<16}", googleads_field.is_repeated)

        # Prints the list of fields that are selectable with the field.
        if googleads_field.selectable_with:
            # Sorts the selectable_with list then prints each field name.
            googleads_field.selectable_with.sort()
            print("  selectable with:")
            selectable_with_field: str
            for selectable_with_field in googleads_field.selectable_with:
                print(f"    {selectable_with_field}")

        # Print an extra line to visually separate each GoogleAdsField.
        print()


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Lists metadata for the specified artifact."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-n",
        "--name_prefix",
        type=str,
        required=True,
        help="The name prefix to use when searching for Google Ads field names",
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(googleads_client, args.name_prefix)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v19.enums.types import local_services_employee_status
from google.ads.googleads.v19.enums.types import local_services_employee_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.resources",
    marshal="google.ads.googleads.v19",
    manifest={
        "LocalServicesEmployee",
        "UniversityDegree",
        "Residency",
        "Fellowship",
    },
)


class LocalServicesEmployee(proto.Message):
    r"""A local services employee resource.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the Local Services
            Verification. Local Services Verification resource names
            have the form:

            ``customers/{customer_id}/localServicesEmployees/{gls_employee_id}``
        id (int):
            Output only. The ID of the employee.

            This field is a member of `oneof`_ ``_id``.
        creation_date_time (str):
            Output only. Timestamp of employee creation.
            The format is "YYYY-MM-DD HH:MM:SS" in the
            Google Ads account's timezone. Examples:
            "2018-03-05 09:15:00" or "2018-02-01 14:34:30".
        status (google.ads.googleads.v19.enums.types.LocalServicesEmployeeStatusEnum.LocalServicesEmployeeStatus):
            Output only. Employee status, such as DELETED
            or ENABLED.
        type_ (google.ads.googleads.v19.enums.types.LocalServicesEmployeeTypeEnum.LocalServicesEmployeeType):
            Output only. Employee type.
        university_degrees (MutableSequence[google.ads.googleads.v19.resources.types.UniversityDegree]):
            Output only. A list of degrees this employee
            has obtained, and wants to feature.
        residencies (MutableSequence[google.ads.googleads.v19.resources.types.Residency]):
            Output only. The institutions where the
            employee has completed their residency.
        fellowships (MutableSequence[google.ads.googleads.v19.resources.types.Fellowship]):
            Output only. The institutions where the
            employee has completed their fellowship.
        job_title (str):
            Output only. Job title for this employee,
            such as "Senior partner" in legal verticals.

            This field is a member of `oneof`_ ``_job_title``.
        year_started_practicing (int):
            Output only. The year that this employee
            started practicing in this field.

            This field is a member of `oneof`_ ``_year_started_practicing``.
        languages_spoken (MutableSequence[str]):
            Output only. Languages that the employee
            speaks, represented as language tags from
            https://developers.google.com/admin-sdk/directory/v1/languages
        category_ids (MutableSequence[str]):
            Output only. Category of the employee. A list of Local
            Services category IDs can be found at
            https://developers.google.com/google-ads/api/data/codes-formats#local_services_ids.
        national_provider_id_number (str):
            Output only. NPI id associated with the
            employee.

            This field is a member of `oneof`_ ``_national_provider_id_number``.
        email_address (str):
            Output only. Email address of the employee.

            This field is a member of `oneof`_ ``_email_address``.
        first_name (str):
            Output only. First name of the employee.

            This field is a member of `oneof`_ ``_first_name``.
        middle_name (str):
            Output only. Middle name of the employee.

            This field is a member of `oneof`_ ``_middle_name``.
        last_name (str):
            Output only. Last name of the employee.

            This field is a member of `oneof`_ ``_last_name``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=2,
        optional=True,
    )
    creation_date_time: str = proto.Field(
        proto.STRING,
        number=3,
    )
    status: (
        local_services_employee_status.LocalServicesEmployeeStatusEnum.LocalServicesEmployeeStatus
    ) = proto.Field(
        proto.ENUM,
        number=4,
        enum=local_services_employee_status.LocalServicesEmployeeStatusEnum.LocalServicesEmployeeStatus,
    )
    type_: (
        local_services_employee_type.LocalServicesEmployeeTypeEnum.LocalServicesEmployeeType
    ) = proto.Field(
        proto.ENUM,
        number=5,
        enum=local_services_employee_type.LocalServicesEmployeeTypeEnum.LocalServicesEmployeeType,
    )
    university_degrees: MutableSequence["UniversityDegree"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=6,
            message="UniversityDegree",
        )
    )
    residencies: MutableSequence["Residency"] = proto.RepeatedField(
        proto.MESSAGE,
        number=7,
        message="Residency",
    )
    fellowships: MutableSequence["Fellowship"] = proto.RepeatedField(
        proto.MESSAGE,
        number=8,
        message="Fellowship",
    )
    job_title: str = proto.Field(
        proto.STRING,
        number=9,
        optional=True,
    )
    year_started_practicing: int = proto.Field(
        proto.INT32,
        number=10,
        optional=True,
    )
    languages_spoken: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=11,
    )
    category_ids: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=12,
    )
    national_provider_id_number: str = proto.Field(
        proto.STRING,
        number=13,
        optional=True,
    )
    email_address: str = proto.Field(
        proto.STRING,
        number=14,
        optional=True,
    )
    first_name: str = proto.Field(
        proto.STRING,
        number=15,
        optional=True,
    )
    middle_name: str = proto.Field(
        proto.STRING,
        number=16,
        optional=True,
    )
    last_name: str = proto.Field(
        proto.STRING,
        number=17,
        optional=True,
    )


class UniversityDegree(proto.Message):
    r"""A list of degrees this employee has obtained, and wants to
    feature.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        institution_name (str):
            Output only. Name of the university at which
            the degree was obtained.

            This field is a member of `oneof`_ ``_institution_name``.
        degree (str):
            Output only. Name of the degree obtained.

            This field is a member of `oneof`_ ``_degree``.
        graduation_year (int):
            Output only. Year of graduation.

            This field is a member of `oneof`_ ``_graduation_year``.
    """

    institution_name: str = proto.Field(
        proto.STRING,
        number=1,
        optional=True,
    )
    degree: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )
    graduation_year: int = proto.Field(
        proto.INT32,
        number=3,
        optional=True,
    )


class Residency(proto.Message):
    r"""Details about the employee's medical residency.
    Residency is a stage of graduate medical education in which a
    qualified medical professional practices under the supervision
    of a senior clinician.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        institution_name (str):
            Output only. Name of the institution at which
            the residency was completed.

            This field is a member of `oneof`_ ``_institution_name``.
        completion_year (int):
            Output only. Year of completion.

            This field is a member of `oneof`_ ``_completion_year``.
    """

    institution_name: str = proto.Field(
        proto.STRING,
        number=1,
        optional=True,
    )
    completion_year: int = proto.Field(
        proto.INT32,
        number=2,
        optional=True,
    )


class Fellowship(proto.Message):
    r"""Details about the employee's medical Fellowship.
    Fellowship is a period of medical training that the professional
    undertakes after finishing their residency.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        institution_name (str):
            Output only. Name of the instutition at which
            the fellowship was completed.

            This field is a member of `oneof`_ ``_institution_name``.
        completion_year (int):
            Output only. Year of completion.

            This field is a member of `oneof`_ ``_completion_year``.
    """

    institution_name: str = proto.Field(
        proto.STRING,
        number=1,
        optional=True,
    )
    completion_year: int = proto.Field(
        proto.INT32,
        number=2,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.errors",
    marshal="google.ads.googleads.v19",
    manifest={
        "AssetLinkErrorEnum",
    },
)


class AssetLinkErrorEnum(proto.Message):
    r"""Container for enum describing possible asset link errors."""

    class AssetLinkError(proto.Enum):
        r"""Enum describing possible asset link errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            PINNING_UNSUPPORTED (2):
                Pinning is not supported for the given asset
                link field.
            UNSUPPORTED_FIELD_TYPE (3):
                The given field type is not supported to be
                added directly through asset links.
            FIELD_TYPE_INCOMPATIBLE_WITH_ASSET_TYPE (4):
                The given asset's type and the specified
                field type are incompatible.
            FIELD_TYPE_INCOMPATIBLE_WITH_CAMPAIGN_TYPE (5):
                The specified field type is incompatible with
                the given campaign type.
            INCOMPATIBLE_ADVERTISING_CHANNEL_TYPE (6):
                The campaign advertising channel type cannot
                be associated with the given asset due to
                channel-based restrictions on the asset's
                fields.
            IMAGE_NOT_WITHIN_SPECIFIED_DIMENSION_RANGE (7):
                The image asset provided is not within the
                dimension constraints specified for the
                submitted asset field.
            INVALID_PINNED_FIELD (8):
                The pinned field is not valid for the
                submitted asset field.
            MEDIA_BUNDLE_ASSET_FILE_SIZE_TOO_LARGE (9):
                The media bundle asset provided is too large
                for the submitted asset field.
            NOT_ENOUGH_AVAILABLE_ASSET_LINKS_FOR_VALID_COMBINATION (10):
                Not enough assets are available for use with
                other fields since other assets are pinned to
                specific fields.
            NOT_ENOUGH_AVAILABLE_ASSET_LINKS_WITH_FALLBACK (11):
                Not enough assets with fallback are
                available. When validating the minimum number of
                assets, assets without fallback (for example,
                assets that contain location tag without default
                value "{LOCATION(City)}") will not be counted.
            NOT_ENOUGH_AVAILABLE_ASSET_LINKS_WITH_FALLBACK_FOR_VALID_COMBINATION (12):
                This is a combination of the
                NOT_ENOUGH_AVAILABLE_ASSET_LINKS_FOR_VALID_COMBINATION and
                NOT_ENOUGH_AVAILABLE_ASSET_LINKS_WITH_FALLBACK errors. Not
                enough assets with fallback are available since some assets
                are pinned.
            YOUTUBE_VIDEO_REMOVED (13):
                The YouTube video referenced in the provided
                asset has been removed.
            YOUTUBE_VIDEO_TOO_LONG (14):
                The YouTube video referenced in the provided
                asset is too long for the field submitted.
            YOUTUBE_VIDEO_TOO_SHORT (15):
                The YouTube video referenced in the provided
                asset is too short for the field submitted.
            EXCLUDED_PARENT_FIELD_TYPE (16):
                The specified field type is excluded for
                given campaign or ad group.
            INVALID_STATUS (17):
                The status is invalid for the operation
                specified.
            YOUTUBE_VIDEO_DURATION_NOT_DEFINED (18):
                The YouTube video referenced in the provided
                asset has unknown duration. This might be the
                case for a livestream video or a video being
                currently uploaded to YouTube. In both cases,
                the video duration should eventually get
                resolved.
            CANNOT_CREATE_AUTOMATICALLY_CREATED_LINKS (19):
                User cannot create automatically created
                links.
            CANNOT_LINK_TO_AUTOMATICALLY_CREATED_ASSET (20):
                Advertiser links cannot link to automatically
                created asset.
            CANNOT_MODIFY_ASSET_LINK_SOURCE (21):
                Automatically created links cannot be changed
                into advertiser links or the reverse.
            CANNOT_LINK_LOCATION_LEAD_FORM_WITHOUT_LOCATION_ASSET (22):
                Lead Form asset with Location answer type
                can't be linked to the Customer/Campaign because
                there are no Location assets.
            CUSTOMER_NOT_VERIFIED (23):
                Customer is not verified.
            UNSUPPORTED_CALL_TO_ACTION (24):
                Call to action value is not supported.
            BRAND_ASSETS_NOT_LINKED_AT_ASSET_GROUP_LEVEL (25):
                For Performance Max campaigns where brand_guidelines_enabled
                is false, business name and logo assets must be linked as
                AssetGroupAssets.
            BRAND_ASSETS_NOT_LINKED_AT_CAMPAIGN_LEVEL (26):
                For Performance Max campaigns where brand_guidelines_enabled
                is true, business name and logo assets must be linked as
                CampaignAssets.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        PINNING_UNSUPPORTED = 2
        UNSUPPORTED_FIELD_TYPE = 3
        FIELD_TYPE_INCOMPATIBLE_WITH_ASSET_TYPE = 4
        FIELD_TYPE_INCOMPATIBLE_WITH_CAMPAIGN_TYPE = 5
        INCOMPATIBLE_ADVERTISING_CHANNEL_TYPE = 6
        IMAGE_NOT_WITHIN_SPECIFIED_DIMENSION_RANGE = 7
        INVALID_PINNED_FIELD = 8
        MEDIA_BUNDLE_ASSET_FILE_SIZE_TOO_LARGE = 9
        NOT_ENOUGH_AVAILABLE_ASSET_LINKS_FOR_VALID_COMBINATION = 10
        NOT_ENOUGH_AVAILABLE_ASSET_LINKS_WITH_FALLBACK = 11
        NOT_ENOUGH_AVAILABLE_ASSET_LINKS_WITH_FALLBACK_FOR_VALID_COMBINATION = (
            12
        )
        YOUTUBE_VIDEO_REMOVED = 13
        YOUTUBE_VIDEO_TOO_LONG = 14
        YOUTUBE_VIDEO_TOO_SHORT = 15
        EXCLUDED_PARENT_FIELD_TYPE = 16
        INVALID_STATUS = 17
        YOUTUBE_VIDEO_DURATION_NOT_DEFINED = 18
        CANNOT_CREATE_AUTOMATICALLY_CREATED_LINKS = 19
        CANNOT_LINK_TO_AUTOMATICALLY_CREATED_ASSET = 20
        CANNOT_MODIFY_ASSET_LINK_SOURCE = 21
        CANNOT_LINK_LOCATION_LEAD_FORM_WITHOUT_LOCATION_ASSET = 22
        CUSTOMER_NOT_VERIFIED = 23
        UNSUPPORTED_CALL_TO_ACTION = 24
        BRAND_ASSETS_NOT_LINKED_AT_ASSET_GROUP_LEVEL = 25
        BRAND_ASSETS_NOT_LINKED_AT_CAMPAIGN_LEVEL = 26


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "PolicyReviewStatusEnum",
    },
)


class PolicyReviewStatusEnum(proto.Message):
    r"""Container for enum describing possible policy review
    statuses.

    """

    class PolicyReviewStatus(proto.Enum):
        r"""The possible policy review statuses.

        Values:
            UNSPECIFIED (0):
                No value has been specified.
            UNKNOWN (1):
                The received value is not known in this
                version.
                This is a response-only value.
            REVIEW_IN_PROGRESS (2):
                Currently under review.
            REVIEWED (3):
                Primary review complete. Other reviews may be
                continuing.
            UNDER_APPEAL (4):
                The resource has been resubmitted for
                approval or its policy decision has been
                appealed.
            ELIGIBLE_MAY_SERVE (5):
                The resource is eligible and may be serving
                but could still undergo further review.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        REVIEW_IN_PROGRESS = 2
        REVIEWED = 3
        UNDER_APPEAL = 4
        ELIGIBLE_MAY_SERVE = 5


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.common.types import asset_policy
from google.ads.googleads.v22.common.types import (
    policy_summary as gagc_policy_summary,
)
from google.ads.googleads.v22.enums.types import asset_field_type
from google.ads.googleads.v22.enums.types import asset_link_primary_status
from google.ads.googleads.v22.enums.types import (
    asset_link_primary_status_reason,
)
from google.ads.googleads.v22.enums.types import asset_link_status
from google.ads.googleads.v22.enums.types import asset_source


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "AssetGroupAsset",
    },
)


class AssetGroupAsset(proto.Message):
    r"""AssetGroupAsset is the link between an asset and an asset
    group. Adding an AssetGroupAsset links an asset with an asset
    group.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the asset group asset. Asset
            group asset resource name have the form:

            ``customers/{customer_id}/assetGroupAssets/{asset_group_id}~{asset_id}~{field_type}``
        asset_group (str):
            Immutable. The asset group which this asset
            group asset is linking.
        asset (str):
            Immutable. The asset which this asset group
            asset is linking.
        field_type (google.ads.googleads.v22.enums.types.AssetFieldTypeEnum.AssetFieldType):
            The description of the placement of the asset within the
            asset group. For example: HEADLINE, YOUTUBE_VIDEO etc
        status (google.ads.googleads.v22.enums.types.AssetLinkStatusEnum.AssetLinkStatus):
            The status of the link between an asset and
            asset group.
        primary_status (google.ads.googleads.v22.enums.types.AssetLinkPrimaryStatusEnum.AssetLinkPrimaryStatus):
            Output only. Provides the PrimaryStatus of
            this asset link. Primary status is meant
            essentially to differentiate between the plain
            "status" field, which has advertiser set values
            of enabled, paused, or removed.  The primary
            status takes into account other signals (for
            assets its mainly policy and quality approvals)
            to come up with a more comprehensive status to
            indicate its serving state.
        primary_status_reasons (MutableSequence[google.ads.googleads.v22.enums.types.AssetLinkPrimaryStatusReasonEnum.AssetLinkPrimaryStatusReason]):
            Output only. Provides a list of reasons for
            why an asset is not serving or not serving at
            full capacity.
        primary_status_details (MutableSequence[google.ads.googleads.v22.common.types.AssetLinkPrimaryStatusDetails]):
            Output only. Provides the details of the
            primary status and its associated reasons.
        policy_summary (google.ads.googleads.v22.common.types.PolicySummary):
            Output only. The policy information for this
            asset group asset.
        source (google.ads.googleads.v22.enums.types.AssetSourceEnum.AssetSource):
            Output only. Source of the asset group asset.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    asset_group: str = proto.Field(
        proto.STRING,
        number=2,
    )
    asset: str = proto.Field(
        proto.STRING,
        number=3,
    )
    field_type: asset_field_type.AssetFieldTypeEnum.AssetFieldType = (
        proto.Field(
            proto.ENUM,
            number=4,
            enum=asset_field_type.AssetFieldTypeEnum.AssetFieldType,
        )
    )
    status: asset_link_status.AssetLinkStatusEnum.AssetLinkStatus = proto.Field(
        proto.ENUM,
        number=5,
        enum=asset_link_status.AssetLinkStatusEnum.AssetLinkStatus,
    )
    primary_status: (
        asset_link_primary_status.AssetLinkPrimaryStatusEnum.AssetLinkPrimaryStatus
    ) = proto.Field(
        proto.ENUM,
        number=8,
        enum=asset_link_primary_status.AssetLinkPrimaryStatusEnum.AssetLinkPrimaryStatus,
    )
    primary_status_reasons: MutableSequence[
        asset_link_primary_status_reason.AssetLinkPrimaryStatusReasonEnum.AssetLinkPrimaryStatusReason
    ] = proto.RepeatedField(
        proto.ENUM,
        number=9,
        enum=asset_link_primary_status_reason.AssetLinkPrimaryStatusReasonEnum.AssetLinkPrimaryStatusReason,
    )
    primary_status_details: MutableSequence[
        asset_policy.AssetLinkPrimaryStatusDetails
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=10,
        message=asset_policy.AssetLinkPrimaryStatusDetails,
    )
    policy_summary: gagc_policy_summary.PolicySummary = proto.Field(
        proto.MESSAGE,
        number=7,
        message=gagc_policy_summary.PolicySummary,
    )
    source: asset_source.AssetSourceEnum.AssetSource = proto.Field(
        proto.ENUM,
        number=11,
        enum=asset_source.AssetSourceEnum.AssetSource,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "PerformanceMaxUpgradeStatusEnum",
    },
)


class PerformanceMaxUpgradeStatusEnum(proto.Message):
    r"""Performance Max Upgrade status for campaign."""

    class PerformanceMaxUpgradeStatus(proto.Enum):
        r"""Performance Max Upgrade status enum for campaign.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            UPGRADE_IN_PROGRESS (3):
                The upgrade to a Performance Max campaign is
                in progress.
            UPGRADE_COMPLETE (4):
                The upgrade to a Performance Max campaign is
                complete.
            UPGRADE_FAILED (5):
                The upgrade to a Performance Max campaign
                failed. The campaign will still serve as it was
                before upgrade was attempted.
            UPGRADE_ELIGIBLE (6):
                The campaign is eligible for upgrade to a
                Performance Max campaign.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        UPGRADE_IN_PROGRESS = 3
        UPGRADE_COMPLETE = 4
        UPGRADE_FAILED = 5
        UPGRADE_ELIGIBLE = 6


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Retrieves the status of the advertiser identity verification program.

If required and not already started, it also starts the verification process.
"""


import argparse
import sys
from typing import Optional

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.enums.types.identity_verification_program_status import (
    IdentityVerificationProgramStatusEnum,
)
from google.ads.googleads.v22.services.services.identity_verification_service.client import (
    IdentityVerificationServiceClient,
)
from google.ads.googleads.v22.services.types.identity_verification_service import (
    GetIdentityVerificationResponse,
    IdentityVerification,
    IdentityVerificationProgress,
)


def main(client: GoogleAdsClient, customer_id: str) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: An initialized GoogleAdsClient instance.
        customer_id: The client customer ID str.
    """
    # Retrieve the current advertiser identity verification status.
    identity_verification: Optional[IdentityVerification] = (
        get_identity_verification(client, customer_id)
    )

    if identity_verification:
        # Type for status is the enum itself, not int, as it's used for direct comparison.
        status: (
            IdentityVerificationProgramStatusEnum.IdentityVerificationProgramStatus
        ) = identity_verification.verification_progress.program_status
        status_enum = (
            client.enums.IdentityVerificationProgramStatusEnum
        )  # This is an EnumTypeWrapper

        if status == status_enum.UNSPECIFIED:
            # Starts an identity verification session.
            start_identity_verification(client, customer_id)
            # Call get_identity_verification again to retrieve the verification
            # progress after starting an identity verification session.
            # The result of this call isn't used here, but if it were, it'd be Optional[IdentityVerification]
            get_identity_verification(client, customer_id)
        elif status == status_enum.PENDING_USER_ACTION:
            # If there is an identity verification session in progress, there
            # is no need to start another one by calling
            # StartIdentityVerification.
            verification_progress: IdentityVerificationProgress = (
                identity_verification.verification_progress
            )
            print(
                "There is an advertiser identity verification session in "
                "progress. The URL for the verification process is: "
                f"{verification_progress.action_url} and it will expire at "
                f"{verification_progress.invitation_link_expiration_time}."
            )
        elif status == status == status_enum.PENDING_REVIEW:
            print("The verification is under review.")
        elif status == status == status_enum.SUCCESS:
            print("The verification completed successfully.")
        else:
            print("The verification has an unknown state.")
    else:
        # If get_identity_verification returned None, the account is not
        # enrolled in mandatory identity verification.
        print(
            f"Account {customer_id} is not required to perform advertiser "
            "identity verification. See "
            "https://support.google.com/adspolicy/answer/9703665 for details "
            "on how and when an account is required to undergo the advertiser "
            "identity verification program."
        )


# [START verify_advertiser_identity_1]
def get_identity_verification(
    client: GoogleAdsClient, customer_id: str
) -> Optional[IdentityVerification]:
    """Retrieves the status of the advertiser identity verification process.

    Args:
        client: An initialized GoogleAdsClient instance.
        customer_id: The client customer ID str.

    Returns:
        either an IdentityVerification instance, or None
    """
    service: IdentityVerificationServiceClient = client.get_service(
        "IdentityVerificationService"
    )
    response: GetIdentityVerificationResponse = (
        service.get_identity_verification(customer_id=customer_id)
    )

    # Check if the response contains any indentity verifications. If not, then
    # None will be returned.
    if response.identity_verification:
        identity_verification_data: IdentityVerification = (
            response.identity_verification[0]
        )
        deadline: str = (
            identity_verification_data.identity_verification_requirement.verification_completion_deadline_time
        )
        # progress is an enum member
        progress: (
            IdentityVerificationProgramStatusEnum.IdentityVerificationProgramStatus
        ) = identity_verification_data.verification_progress.program_status

        print(
            f"Account {customer_id} has a verification completion deadline "
            "of {deadline} and status {progress.name} for advertiser identity "  # Use .name for string representation of enum
            "verification."
        )

        return identity_verification_data
        # [END verify_advertiser_identity_1]
    return None  # Explicitly return None if no identity_verification found


# [START verify_advertiser_identity_2]
def start_identity_verification(
    client: GoogleAdsClient, customer_id: str
) -> None:
    """Starts the identity verification process.

    Args:
        client: An initialized GoogleAdsClient instance.
        customer_id: The client customer ID str.
    """
    service: IdentityVerificationServiceClient = client.get_service(
        "IdentityVerificationService"
    )
    # Sends a request to start the identity verification process.
    # The verification_program argument expects an IdentityVerificationProgramEnum value (int).
    service.start_identity_verification(
        customer_id=customer_id,
        verification_program=client.enums.IdentityVerificationProgramEnum.ADVERTISER_IDENTITY_VERIFICATION.value,
    )
    # [END verify_advertiser_identity_2]


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description=(
            "Retrieves the status of the advertiser identity verification "
            "program. If required and not already started, it also starts the "
            "verification process."
        )
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    args = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client = GoogleAdsClient.load_from_storage(version="v22")

    try:
        main(googleads_client, args.customer_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.errors",
    marshal="google.ads.googleads.v22",
    manifest={
        "CustomAudienceErrorEnum",
    },
)


class CustomAudienceErrorEnum(proto.Message):
    r"""Container for enum describing possible custom audience
    errors.

    """

    class CustomAudienceError(proto.Enum):
        r"""Enum describing possible custom audience errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            NAME_ALREADY_USED (2):
                New name in the custom audience is duplicated
                ignoring cases.
            CANNOT_REMOVE_WHILE_IN_USE (3):
                Cannot remove a custom audience while it's
                still being used as targeting.
            RESOURCE_ALREADY_REMOVED (4):
                Cannot update or remove a custom audience
                that is already removed.
            MEMBER_TYPE_AND_PARAMETER_ALREADY_EXISTED (5):
                The pair of [type, value] already exists in members.
            INVALID_MEMBER_TYPE (6):
                Member type is invalid.
            MEMBER_TYPE_AND_VALUE_DOES_NOT_MATCH (7):
                Member type does not have associated value.
            POLICY_VIOLATION (8):
                Custom audience contains a member that
                violates policy.
            INVALID_TYPE_CHANGE (9):
                Change in custom audience type is not
                allowed.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        NAME_ALREADY_USED = 2
        CANNOT_REMOVE_WHILE_IN_USE = 3
        RESOURCE_ALREADY_REMOVED = 4
        MEMBER_TYPE_AND_PARAMETER_ALREADY_EXISTED = 5
        INVALID_MEMBER_TYPE = 6
        MEMBER_TYPE_AND_VALUE_DOES_NOT_MATCH = 7
        POLICY_VIOLATION = 8
        INVALID_TYPE_CHANGE = 9


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "AssetGroupSignalErrorEnum",
    },
)


class AssetGroupSignalErrorEnum(proto.Message):
    r"""Container for enum describing possible asset group signal
    errors.

    """

    class AssetGroupSignalError(proto.Enum):
        r"""Enum describing possible asset group signal errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            TOO_MANY_WORDS (2):
                The number of words in the Search Theme
                signal exceed the allowed maximum. You can add
                up to 10 words in a keyword. See
                https://support.google.com/google-ads/answer/7476658
                for details.
            SEARCH_THEME_POLICY_VIOLATION (3):
                The search theme requested to be added
                violates certain policy. See
                https://support.google.com/adspolicy/answer/6008942.
            AUDIENCE_WITH_WRONG_ASSET_GROUP_ID (4):
                The asset group referenced by the asset group
                signal does not match the asset group referenced
                by the audience being used in the asset group
                signal.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        TOO_MANY_WORDS = 2
        SEARCH_THEME_POLICY_VIOLATION = 3
        AUDIENCE_WITH_WRONG_ASSET_GROUP_ID = 4


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example generates forecast metrics for keyword planning.

For more details see this guide:
https://developers.google.com/google-ads/api/docs/keyword-planning/generate-forecast-metrics
"""

import argparse
from datetime import datetime, timedelta
import sys

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.common.types.criteria import (
    KeywordInfo,
)
from google.ads.googleads.v22.services.services.google_ads_service.client import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.services.keyword_plan_idea_service.client import (
    KeywordPlanIdeaServiceClient,
)
from google.ads.googleads.v22.services.types.keyword_plan_idea_service import (
    CampaignToForecast,
    CriterionBidModifier,
    ForecastAdGroup,
    BiddableKeyword,
    GenerateKeywordForecastMetricsRequest,
    GenerateKeywordForecastMetricsResponse,
)


# [START generate_forecast_metrics]
def main(client: GoogleAdsClient, customer_id: str):
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
    """
    campaign_to_forecast: CampaignToForecast = create_campaign_to_forecast(
        client
    )
    generate_forecast_metrics(client, customer_id, campaign_to_forecast)


def create_campaign_to_forecast(client: GoogleAdsClient) -> CampaignToForecast:
    """Creates the campaign to forecast.

    A campaign to forecast lets you try out various configurations and keywords
    to find the best optimization for your future campaigns. Once you've found
    the best campaign configuration, create a serving campaign in your Google
    Ads account with similar values and keywords. For more details, see:
    https://support.google.com/google-ads/answer/3022575

    Args:
        client: an initialized GoogleAdsClient instance.

    Returns:
        An CampaignToForecast instance.
    """
    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )
    # Create a campaign to forecast.
    campaign_to_forecast: CampaignToForecast = client.get_type(
        "CampaignToForecast"
    )
    campaign_to_forecast.keyword_plan_network = (
        client.enums.KeywordPlanNetworkEnum.GOOGLE_SEARCH
    )

    # Set the bidding strategy.
    campaign_to_forecast.bidding_strategy.manual_cpc_bidding_strategy.max_cpc_bid_micros = (
        1000000
    )

    # For the list of geo target IDs, see:
    # https://developers.google.com/google-ads/api/reference/data/geotargets
    criterion_bid_modifier: CriterionBidModifier = client.get_type(
        "CriterionBidModifier"
    )
    # Geo target constant 2840 is for USA.
    criterion_bid_modifier.geo_target_constant = (
        googleads_service.geo_target_constant_path("2840")
    )
    campaign_to_forecast.geo_modifiers.append(criterion_bid_modifier)

    # For the list of language criteria IDs, see:
    # https://developers.google.com/google-ads/api/reference/data/codes-formats#languages
    # Language criteria 1000 is for English.
    campaign_to_forecast.language_constants.append(
        googleads_service.language_constant_path("1000")
    )

    # Create forecast ad groups based on themes such as creative relevance,
    # product category, or cost per click.
    forecast_ad_group: ForecastAdGroup = client.get_type("ForecastAdGroup")

    # Create and configure three BiddableKeyword instances.
    biddable_keyword_1: BiddableKeyword = client.get_type("BiddableKeyword")
    biddable_keyword_1.max_cpc_bid_micros = 2500000
    biddable_keyword_1.keyword.text = "mars cruise"
    biddable_keyword_1.keyword.match_type = (
        client.enums.KeywordMatchTypeEnum.BROAD
    )

    biddable_keyword_2: BiddableKeyword = client.get_type("BiddableKeyword")
    biddable_keyword_2.max_cpc_bid_micros = 1500000
    biddable_keyword_2.keyword.text = "cheap cruise"
    biddable_keyword_2.keyword.match_type = (
        client.enums.KeywordMatchTypeEnum.PHRASE
    )

    biddable_keyword_3: BiddableKeyword = client.get_type("BiddableKeyword")
    biddable_keyword_3.max_cpc_bid_micros = 1990000
    biddable_keyword_3.keyword.text = "cheap cruise"
    biddable_keyword_3.keyword.match_type = (
        client.enums.KeywordMatchTypeEnum.EXACT
    )

    # Add the biddable keywords to the forecast ad group.
    forecast_ad_group.biddable_keywords.extend(
        [biddable_keyword_1, biddable_keyword_2, biddable_keyword_3]
    )

    # Create and configure a negative keyword, then add it to the forecast ad
    # group.
    negative_keyword: KeywordInfo = client.get_type("KeywordInfo")
    negative_keyword.text = "moon walk"
    negative_keyword.match_type = client.enums.KeywordMatchTypeEnum.BROAD
    forecast_ad_group.negative_keywords.append(negative_keyword)

    campaign_to_forecast.ad_groups.append(forecast_ad_group)

    return campaign_to_forecast


def generate_forecast_metrics(
    client: GoogleAdsClient,
    customer_id: str,
    campaign_to_forecast: CampaignToForecast,
):
    """Generates forecast metrics and prints the results.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        campaign_to_forecast: a CampaignToForecast to generate metrics for.
    """
    keyword_plan_idea_service: KeywordPlanIdeaServiceClient = (
        client.get_service("KeywordPlanIdeaService")
    )
    request: GenerateKeywordForecastMetricsRequest = client.get_type(
        "GenerateKeywordForecastMetricsRequest"
    )
    request.customer_id = customer_id
    request.campaign = campaign_to_forecast
    # Set the forecast range. Repeat forecasts with different horizons to get a
    # holistic picture.
    # Set the forecast start date to tomorrow.
    tomorrow = datetime.now() + timedelta(days=1)
    request.forecast_period.start_date = tomorrow.strftime("%Y-%m-%d")
    # Set the forecast end date to 30 days from today.
    thirty_days_from_now = datetime.now() + timedelta(days=30)
    request.forecast_period.end_date = thirty_days_from_now.strftime("%Y-%m-%d")

    response: GenerateKeywordForecastMetricsResponse = (
        keyword_plan_idea_service.generate_keyword_forecast_metrics(
            request=request
        )
    )

    metrics = response.campaign_forecast_metrics
    print(f"Estimated daily clicks: {metrics.clicks}")
    print(f"Estimated daily impressions: {metrics.impressions}")
    print(f"Estimated daily average CPC: {metrics.average_cpc_micros}")
    # [END generate_forecast_metrics]


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Generates forecast metrics for keyword planning."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )

    args = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client = GoogleAdsClient.load_from_storage(version="v22")

    try:
        main(googleads_client, args.customer_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'Error with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import abc
from typing import Awaitable, Callable, Optional, Sequence, Union

from google.ads.googleads.v21 import gapic_version as package_version

import google.auth  # type: ignore
import google.api_core
from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore
import google.protobuf

from google.ads.googleads.v21.services.types import ad_group_criterion_service

DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)

if hasattr(DEFAULT_CLIENT_INFO, "protobuf_runtime_version"):  # pragma: NO COVER
    DEFAULT_CLIENT_INFO.protobuf_runtime_version = google.protobuf.__version__


class AdGroupCriterionServiceTransport(abc.ABC):
    """Abstract transport class for AdGroupCriterionService."""

    AUTH_SCOPES = ("https://www.googleapis.com/auth/adwords",)

    DEFAULT_HOST: str = "googleads.googleapis.com"

    def __init__(
        self,
        *,
        host: str = DEFAULT_HOST,
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        api_audience: Optional[str] = None,
        **kwargs,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to (default: 'googleads.googleapis.com').
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is mutually exclusive with credentials.
            scopes (Optional[Sequence[str]]): A list of scopes.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.
        """

        scopes_kwargs = {"scopes": scopes, "default_scopes": self.AUTH_SCOPES}

        # Save the scopes.
        self._scopes = scopes
        if not hasattr(self, "_ignore_credentials"):
            self._ignore_credentials: bool = False

        # If no credentials are provided, then determine the appropriate
        # defaults.
        if credentials and credentials_file:
            raise core_exceptions.DuplicateCredentialArgs(
                "'credentials_file' and 'credentials' are mutually exclusive"
            )

        if credentials_file is not None:
            credentials, _ = google.auth.load_credentials_from_file(
                credentials_file,
                **scopes_kwargs,
                quota_project_id=quota_project_id,
            )
        elif credentials is None and not self._ignore_credentials:
            credentials, _ = google.auth.default(
                **scopes_kwargs, quota_project_id=quota_project_id
            )
            # Don't apply audience if the credentials file passed from user.
            if hasattr(credentials, "with_gdch_audience"):
                credentials = credentials.with_gdch_audience(
                    api_audience if api_audience else host
                )

        # If the credentials are service account credentials, then always try to use self signed JWT.
        if (
            always_use_jwt_access
            and isinstance(credentials, service_account.Credentials)
            and hasattr(
                service_account.Credentials, "with_always_use_jwt_access"
            )
        ):
            credentials = credentials.with_always_use_jwt_access(True)

        # Save the credentials.
        self._credentials = credentials

        # Save the hostname. Default to port 443 (HTTPS) if none is specified.
        if ":" not in host:
            host += ":443"
        self._host = host

    @property
    def host(self):
        return self._host

    def _prep_wrapped_messages(self, client_info):
        # Precompute the wrapped methods.
        self._wrapped_methods = {
            self.mutate_ad_group_criteria: gapic_v1.method.wrap_method(
                self.mutate_ad_group_criteria,
                default_timeout=None,
                client_info=client_info,
            ),
        }

    def close(self):
        """Closes resources associated with the transport.

        .. warning::
             Only call this method if the transport is NOT shared
             with other clients - this may cause errors in other clients!
        """
        raise NotImplementedError()

    @property
    def mutate_ad_group_criteria(
        self,
    ) -> Callable[
        [ad_group_criterion_service.MutateAdGroupCriteriaRequest],
        Union[
            ad_group_criterion_service.MutateAdGroupCriteriaResponse,
            Awaitable[ad_group_criterion_service.MutateAdGroupCriteriaResponse],
        ],
    ]:
        raise NotImplementedError()

    @property
    def kind(self) -> str:
        raise NotImplementedError()


__all__ = ("AdGroupCriterionServiceTransport",)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v20.common.types import policy
from google.ads.googleads.v20.enums.types import asset_field_type
from google.ads.googleads.v20.enums.types import asset_performance_label
from google.ads.googleads.v20.enums.types import asset_source
from google.ads.googleads.v20.enums.types import policy_approval_status
from google.ads.googleads.v20.enums.types import policy_review_status
from google.ads.googleads.v20.enums.types import served_asset_field_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "AdGroupAdAssetView",
        "AdGroupAdAssetPolicySummary",
    },
)


class AdGroupAdAssetView(proto.Message):
    r"""A link between an AdGroupAd and an Asset. AdGroupAdAssetView
    supports AppAds, Demand Gen campaigns, and Responsive Search
    Ads.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the ad group ad asset
            view. Ad group ad asset view resource names have the form
            (Before V4):

            ``customers/{customer_id}/adGroupAdAssets/{AdGroupAdAsset.ad_group_id}~{AdGroupAdAsset.ad.ad_id}~{AdGroupAdAsset.asset_id}~{AdGroupAdAsset.field_type}``

            Ad group ad asset view resource names have the form
            (Beginning from V4):

            ``customers/{customer_id}/adGroupAdAssetViews/{AdGroupAdAsset.ad_group_id}~{AdGroupAdAsset.ad_id}~{AdGroupAdAsset.asset_id}~{AdGroupAdAsset.field_type}``
        ad_group_ad (str):
            Output only. The ad group ad to which the
            asset is linked.

            This field is a member of `oneof`_ ``_ad_group_ad``.
        asset (str):
            Output only. The asset which is linked to the
            ad group ad.

            This field is a member of `oneof`_ ``_asset``.
        field_type (google.ads.googleads.v20.enums.types.AssetFieldTypeEnum.AssetFieldType):
            Output only. Role that the asset takes in the
            ad.
        enabled (bool):
            Output only. The status between the asset and
            the latest version of the ad. If true, the asset
            is linked to the latest version of the ad. If
            false, it means the link once existed but has
            been removed and is no longer present in the
            latest version of the ad.

            This field is a member of `oneof`_ ``_enabled``.
        policy_summary (google.ads.googleads.v20.resources.types.AdGroupAdAssetPolicySummary):
            Output only. Policy information for the ad
            group ad asset.
        performance_label (google.ads.googleads.v20.enums.types.AssetPerformanceLabelEnum.AssetPerformanceLabel):
            Output only. Performance of an asset linkage.
        pinned_field (google.ads.googleads.v20.enums.types.ServedAssetFieldTypeEnum.ServedAssetFieldType):
            Output only. Pinned field.
        source (google.ads.googleads.v20.enums.types.AssetSourceEnum.AssetSource):
            Output only. Source of the ad group ad asset.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    ad_group_ad: str = proto.Field(
        proto.STRING,
        number=9,
        optional=True,
    )
    asset: str = proto.Field(
        proto.STRING,
        number=10,
        optional=True,
    )
    field_type: asset_field_type.AssetFieldTypeEnum.AssetFieldType = (
        proto.Field(
            proto.ENUM,
            number=2,
            enum=asset_field_type.AssetFieldTypeEnum.AssetFieldType,
        )
    )
    enabled: bool = proto.Field(
        proto.BOOL,
        number=8,
        optional=True,
    )
    policy_summary: "AdGroupAdAssetPolicySummary" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="AdGroupAdAssetPolicySummary",
    )
    performance_label: (
        asset_performance_label.AssetPerformanceLabelEnum.AssetPerformanceLabel
    ) = proto.Field(
        proto.ENUM,
        number=4,
        enum=asset_performance_label.AssetPerformanceLabelEnum.AssetPerformanceLabel,
    )
    pinned_field: (
        served_asset_field_type.ServedAssetFieldTypeEnum.ServedAssetFieldType
    ) = proto.Field(
        proto.ENUM,
        number=11,
        enum=served_asset_field_type.ServedAssetFieldTypeEnum.ServedAssetFieldType,
    )
    source: asset_source.AssetSourceEnum.AssetSource = proto.Field(
        proto.ENUM,
        number=12,
        enum=asset_source.AssetSourceEnum.AssetSource,
    )


class AdGroupAdAssetPolicySummary(proto.Message):
    r"""Contains policy information for an ad group ad asset.

    Attributes:
        policy_topic_entries (MutableSequence[google.ads.googleads.v20.common.types.PolicyTopicEntry]):
            Output only. The list of policy findings for
            the ad group ad asset.
        review_status (google.ads.googleads.v20.enums.types.PolicyReviewStatusEnum.PolicyReviewStatus):
            Output only. Where in the review process this
            ad group ad asset is.
        approval_status (google.ads.googleads.v20.enums.types.PolicyApprovalStatusEnum.PolicyApprovalStatus):
            Output only. The overall approval status of
            this ad group ad asset, calculated based on the
            status of its individual policy topic entries.
    """

    policy_topic_entries: MutableSequence[policy.PolicyTopicEntry] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=policy.PolicyTopicEntry,
        )
    )
    review_status: (
        policy_review_status.PolicyReviewStatusEnum.PolicyReviewStatus
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=policy_review_status.PolicyReviewStatusEnum.PolicyReviewStatus,
    )
    approval_status: (
        policy_approval_status.PolicyApprovalStatusEnum.PolicyApprovalStatus
    ) = proto.Field(
        proto.ENUM,
        number=3,
        enum=policy_approval_status.PolicyApprovalStatusEnum.PolicyApprovalStatus,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from copy import deepcopy
from google.ads.googleads.util import (
    set_nested_message_field,
    get_nested_attr,
    convert_proto_plus_to_protobuf,
    proto_copy_from,
)

# The keys in this dict represent messages that have fields that may contain
# sensitive information, such as PII like an email address, that shouldn't
# be logged. The values are a list of dot-delimited field paths on the message
# where the sensitive information may exist. Messages listed here will have
# their sensitive fields redacted in logs when transmitted in the following
# scenarios:
#     1. They are returned as part of a Search or SearchStream request.
#     2. They are returned individually in a Get request.
#     3. They are sent to the API as part of a Mutate request.
_MESSAGES_WITH_SENSITIVE_FIELDS = {
    "CustomerUserAccess": ["email_address", "inviter_user_email_address"],
    "CustomerUserAccessInvitation": ["email_address"],
    "MutateCustomerUserAccessRequest": [
        "operation.update.email_address",
        "operation.update.inviter_user_email_address",
    ],
    "ChangeEvent": ["user_email"],
    "CreateCustomerClientRequest": ["email_address"],
    "Feed": ["places_location_feed_data.email_address"],
    "LocalServicesLead": [
        "contact_details.phone_number",
        "contact_details.email",
        "contact_details.consumer_name",
    ],
    "LocalServicesLeadConversation": ["message_details.text"],
}

# This is a list of the names of messages that return search results from the
# API. These messages contain other messages that may contain sensitive
# information that needs to be masked before being logged.
_SEARCH_RESPONSE_MESSAGE_NAMES = [
    "SearchGoogleAdsResponse",
    "SearchGoogleAdsStreamResponse",
]


def _copy_message(message):
    """Returns a copy of the given message.

    Args:
        message: An object containing information from an API request
            or response.

    Returns:
        A copy of the given message.
    """
    return deepcopy(message)


def _mask_message_fields(field_list, message, mask):
    """Copies the given message and masks sensitive fields.

    Sensitive fields are given as a list of strings and are overridden
    with the word "REDACTED" to protect PII from being logged.

    Args:
        field_list: A list of strings specifying the fields on the message
            that should be masked.
        message: An object containing information from an API request
            or response.
        mask: A str that should replace the sensitive information in the
            message.

    Returns:
        A new instance of the message object with fields copied and masked
            where necessary.
    """
    copy = _copy_message(message)

    for field_path in field_list:
        try:
            # Only mask the field if it's been set on the message.
            if get_nested_attr(copy, field_path):
                set_nested_message_field(copy, field_path, mask)
        except AttributeError:
            # AttributeError is raised when the field is not defined on the
            # message. In this case there's nothing to mask and the field
            # should be skipped.
            break

    return copy


def _mask_google_ads_search_response(message, mask):
    """Copies and masks sensitive data in a Search response

    Response messages include instances of GoogleAdsSearchResponse and
    GoogleAdsSearchStreamResponse.

    Args:
        message: A SearchGoogleAdsResponse or SearchGoogleAdsStreamResponse
            instance.
        mask: A str that should replace the sensitive information in the
            message.

    Returns:
        A copy of the message with sensitive fields masked.
    """
    copy = _copy_message(message)

    for row in copy.results:
        # Each row is an instance of GoogleAdsRow. The ListFields method
        # returns a list of (FieldDescriptor, value) tuples for all fields in
        # the message which are not empty. If this message is a proto_plus proto
        # then we need to access the native proto to call ListFields. If it's
        # not proto_plus we can assume it's protobuf and can access ListFields
        # directly.
        if hasattr(row, "_pb"):
            row_fields = convert_proto_plus_to_protobuf(row).ListFields()
        else:
            row_fields = row.ListFields()
        for field in row_fields:
            field_descriptor = field[0]
            # field_name is the name of the field on the GoogleAdsRow instance,
            # for example "campaign" or "customer_user_access"
            field_name = field_descriptor.name
            # message_name is the name of the message, similar to the class
            # name, for example "Campaign" or "CustomerUserAccess"
            message_name = field_descriptor.message_type.name
            if message_name in _MESSAGES_WITH_SENSITIVE_FIELDS.keys():
                nested_message = getattr(row, field_name)
                masked_message = _mask_message_fields(
                    _MESSAGES_WITH_SENSITIVE_FIELDS[message_name],
                    nested_message,
                    mask,
                )
                # Overwrites the nested message with an exact copy of itself,
                # where sensitive fields have been masked.
                proto_copy_from(getattr(row, field_name), masked_message)

    return copy


def mask_message(message, mask):
    """Copies and returns a message with sensitive fields masked.

    Args:
        message: An object containing information from an API request
            or response.
        mask: A str that should replace the sensitive information in the
            message.

    Returns:
        A copy of the message instance with sensitive fields masked.
    """
    class_name = message.__class__.__name__

    if class_name in _SEARCH_RESPONSE_MESSAGE_NAMES:
        return _mask_google_ads_search_response(message, mask)
    elif class_name in _MESSAGES_WITH_SENSITIVE_FIELDS.keys():
        sensitive_fields = _MESSAGES_WITH_SENSITIVE_FIELDS[class_name]
        return _mask_message_fields(sensitive_fields, message, mask)
    else:
        return message



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v19.common.types import lifecycle_goals
from google.ads.googleads.v19.enums.types import (
    customer_acquisition_optimization_mode,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.resources",
    marshal="google.ads.googleads.v19",
    manifest={
        "CampaignLifecycleGoal",
        "CustomerAcquisitionGoalSettings",
    },
)


class CampaignLifecycleGoal(proto.Message):
    r"""Campaign level customer lifecycle goal settings.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the customer lifecycle goal
            of a campaign.

            ``customers/{customer_id}/campaignLifecycleGoal/{campaign_id}``
        campaign (str):
            Output only. The campaign where the goal is
            attached.
        customer_acquisition_goal_settings (google.ads.googleads.v19.resources.types.CustomerAcquisitionGoalSettings):
            Output only. The customer acquisition goal
            settings for the campaign. The customer
            acquisition goal is described in this article:

            https://support.google.com/google-ads/answer/12080169
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    campaign: str = proto.Field(
        proto.STRING,
        number=2,
    )
    customer_acquisition_goal_settings: "CustomerAcquisitionGoalSettings" = (
        proto.Field(
            proto.MESSAGE,
            number=3,
            message="CustomerAcquisitionGoalSettings",
        )
    )


class CustomerAcquisitionGoalSettings(proto.Message):
    r"""The customer acquisition goal settings for the campaign.

    Attributes:
        optimization_mode (google.ads.googleads.v19.enums.types.CustomerAcquisitionOptimizationModeEnum.CustomerAcquisitionOptimizationMode):
            Output only. Customer acquisition
            optimization mode of this campaign.
        value_settings (google.ads.googleads.v19.common.types.LifecycleGoalValueSettings):
            Output only. Campaign specific values for the
            customer acquisition goal.
    """

    optimization_mode: (
        customer_acquisition_optimization_mode.CustomerAcquisitionOptimizationModeEnum.CustomerAcquisitionOptimizationMode
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=customer_acquisition_optimization_mode.CustomerAcquisitionOptimizationModeEnum.CustomerAcquisitionOptimizationMode,
    )
    value_settings: lifecycle_goals.LifecycleGoalValueSettings = proto.Field(
        proto.MESSAGE,
        number=2,
        message=lifecycle_goals.LifecycleGoalValueSettings,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.common.types import asset_types
from google.ads.googleads.v21.common.types import custom_parameter
from google.ads.googleads.v21.common.types import policy
from google.ads.googleads.v21.enums.types import (
    asset_field_type as gage_asset_field_type,
)
from google.ads.googleads.v21.enums.types import (
    asset_source as gage_asset_source,
)
from google.ads.googleads.v21.enums.types import asset_type
from google.ads.googleads.v21.enums.types import policy_approval_status
from google.ads.googleads.v21.enums.types import policy_review_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.resources",
    marshal="google.ads.googleads.v21",
    manifest={
        "Asset",
        "AssetFieldTypePolicySummary",
        "AssetPolicySummary",
    },
)


class Asset(proto.Message):
    r"""Asset is a part of an ad which can be shared across multiple
    ads. It can be an image (ImageAsset), a video
    (YoutubeVideoAsset), etc. Assets are immutable and cannot be
    removed. To stop an asset from serving, remove the asset from
    the entity that is using it.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the asset. Asset resource
            names have the form:

            ``customers/{customer_id}/assets/{asset_id}``
        id (int):
            Output only. The ID of the asset.

            This field is a member of `oneof`_ ``_id``.
        name (str):
            Optional name of the asset.

            This field is a member of `oneof`_ ``_name``.
        type_ (google.ads.googleads.v21.enums.types.AssetTypeEnum.AssetType):
            Output only. Type of the asset.
        final_urls (MutableSequence[str]):
            A list of possible final URLs after all cross
            domain redirects.
        final_mobile_urls (MutableSequence[str]):
            A list of possible final mobile URLs after
            all cross domain redirects.
        tracking_url_template (str):
            URL template for constructing a tracking URL.

            This field is a member of `oneof`_ ``_tracking_url_template``.
        url_custom_parameters (MutableSequence[google.ads.googleads.v21.common.types.CustomParameter]):
            A list of mappings to be used for substituting URL custom
            parameter tags in the tracking_url_template, final_urls,
            and/or final_mobile_urls.
        final_url_suffix (str):
            URL template for appending params to landing
            page URLs served with parallel tracking.

            This field is a member of `oneof`_ ``_final_url_suffix``.
        source (google.ads.googleads.v21.enums.types.AssetSourceEnum.AssetSource):
            Output only. Source of the asset.
        policy_summary (google.ads.googleads.v21.resources.types.AssetPolicySummary):
            Output only. Policy information for the
            asset.
        field_type_policy_summaries (MutableSequence[google.ads.googleads.v21.resources.types.AssetFieldTypePolicySummary]):
            Output only. Policy information for the asset
            for each FieldType.
        youtube_video_asset (google.ads.googleads.v21.common.types.YoutubeVideoAsset):
            Immutable. A YouTube video asset.

            This field is a member of `oneof`_ ``asset_data``.
        media_bundle_asset (google.ads.googleads.v21.common.types.MediaBundleAsset):
            Immutable. A media bundle asset.

            This field is a member of `oneof`_ ``asset_data``.
        image_asset (google.ads.googleads.v21.common.types.ImageAsset):
            Output only. An image asset.

            This field is a member of `oneof`_ ``asset_data``.
        text_asset (google.ads.googleads.v21.common.types.TextAsset):
            Immutable. A text asset.

            This field is a member of `oneof`_ ``asset_data``.
        lead_form_asset (google.ads.googleads.v21.common.types.LeadFormAsset):
            A lead form asset.

            This field is a member of `oneof`_ ``asset_data``.
        book_on_google_asset (google.ads.googleads.v21.common.types.BookOnGoogleAsset):
            A book on google asset.

            This field is a member of `oneof`_ ``asset_data``.
        promotion_asset (google.ads.googleads.v21.common.types.PromotionAsset):
            A promotion asset.

            This field is a member of `oneof`_ ``asset_data``.
        callout_asset (google.ads.googleads.v21.common.types.CalloutAsset):
            A callout asset.

            This field is a member of `oneof`_ ``asset_data``.
        structured_snippet_asset (google.ads.googleads.v21.common.types.StructuredSnippetAsset):
            A structured snippet asset.

            This field is a member of `oneof`_ ``asset_data``.
        sitelink_asset (google.ads.googleads.v21.common.types.SitelinkAsset):
            A sitelink asset.

            This field is a member of `oneof`_ ``asset_data``.
        page_feed_asset (google.ads.googleads.v21.common.types.PageFeedAsset):
            A page feed asset.

            This field is a member of `oneof`_ ``asset_data``.
        dynamic_education_asset (google.ads.googleads.v21.common.types.DynamicEducationAsset):
            A dynamic education asset.

            This field is a member of `oneof`_ ``asset_data``.
        mobile_app_asset (google.ads.googleads.v21.common.types.MobileAppAsset):
            A mobile app asset.

            This field is a member of `oneof`_ ``asset_data``.
        hotel_callout_asset (google.ads.googleads.v21.common.types.HotelCalloutAsset):
            A hotel callout asset.

            This field is a member of `oneof`_ ``asset_data``.
        call_asset (google.ads.googleads.v21.common.types.CallAsset):
            A call asset.

            This field is a member of `oneof`_ ``asset_data``.
        price_asset (google.ads.googleads.v21.common.types.PriceAsset):
            A price asset.

            This field is a member of `oneof`_ ``asset_data``.
        call_to_action_asset (google.ads.googleads.v21.common.types.CallToActionAsset):
            Immutable. A call to action asset.

            This field is a member of `oneof`_ ``asset_data``.
        dynamic_real_estate_asset (google.ads.googleads.v21.common.types.DynamicRealEstateAsset):
            A dynamic real estate asset.

            This field is a member of `oneof`_ ``asset_data``.
        dynamic_custom_asset (google.ads.googleads.v21.common.types.DynamicCustomAsset):
            A dynamic custom asset.

            This field is a member of `oneof`_ ``asset_data``.
        dynamic_hotels_and_rentals_asset (google.ads.googleads.v21.common.types.DynamicHotelsAndRentalsAsset):
            A dynamic hotels and rentals asset.

            This field is a member of `oneof`_ ``asset_data``.
        dynamic_flights_asset (google.ads.googleads.v21.common.types.DynamicFlightsAsset):
            A dynamic flights asset.

            This field is a member of `oneof`_ ``asset_data``.
        demand_gen_carousel_card_asset (google.ads.googleads.v21.common.types.DemandGenCarouselCardAsset):
            Immutable. A Demand Gen carousel card asset.

            This field is a member of `oneof`_ ``asset_data``.
        dynamic_travel_asset (google.ads.googleads.v21.common.types.DynamicTravelAsset):
            A dynamic travel asset.

            This field is a member of `oneof`_ ``asset_data``.
        dynamic_local_asset (google.ads.googleads.v21.common.types.DynamicLocalAsset):
            A dynamic local asset.

            This field is a member of `oneof`_ ``asset_data``.
        dynamic_jobs_asset (google.ads.googleads.v21.common.types.DynamicJobsAsset):
            A dynamic jobs asset.

            This field is a member of `oneof`_ ``asset_data``.
        location_asset (google.ads.googleads.v21.common.types.LocationAsset):
            Output only. A location asset.

            This field is a member of `oneof`_ ``asset_data``.
        hotel_property_asset (google.ads.googleads.v21.common.types.HotelPropertyAsset):
            Immutable. A hotel property asset.

            This field is a member of `oneof`_ ``asset_data``.
        business_message_asset (google.ads.googleads.v21.common.types.BusinessMessageAsset):
            A business message asset.

            This field is a member of `oneof`_ ``asset_data``.
        app_deep_link_asset (google.ads.googleads.v21.common.types.AppDeepLinkAsset):
            Immutable. An app deep link asset.

            This field is a member of `oneof`_ ``asset_data``.
        youtube_video_list_asset (google.ads.googleads.v21.common.types.YouTubeVideoListAsset):
            Immutable. A YouTube video list asset.

            This field is a member of `oneof`_ ``asset_data``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=11,
        optional=True,
    )
    name: str = proto.Field(
        proto.STRING,
        number=12,
        optional=True,
    )
    type_: asset_type.AssetTypeEnum.AssetType = proto.Field(
        proto.ENUM,
        number=4,
        enum=asset_type.AssetTypeEnum.AssetType,
    )
    final_urls: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=14,
    )
    final_mobile_urls: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=16,
    )
    tracking_url_template: str = proto.Field(
        proto.STRING,
        number=17,
        optional=True,
    )
    url_custom_parameters: MutableSequence[custom_parameter.CustomParameter] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=18,
            message=custom_parameter.CustomParameter,
        )
    )
    final_url_suffix: str = proto.Field(
        proto.STRING,
        number=19,
        optional=True,
    )
    source: gage_asset_source.AssetSourceEnum.AssetSource = proto.Field(
        proto.ENUM,
        number=38,
        enum=gage_asset_source.AssetSourceEnum.AssetSource,
    )
    policy_summary: "AssetPolicySummary" = proto.Field(
        proto.MESSAGE,
        number=13,
        message="AssetPolicySummary",
    )
    field_type_policy_summaries: MutableSequence[
        "AssetFieldTypePolicySummary"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=40,
        message="AssetFieldTypePolicySummary",
    )
    youtube_video_asset: asset_types.YoutubeVideoAsset = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="asset_data",
        message=asset_types.YoutubeVideoAsset,
    )
    media_bundle_asset: asset_types.MediaBundleAsset = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="asset_data",
        message=asset_types.MediaBundleAsset,
    )
    image_asset: asset_types.ImageAsset = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="asset_data",
        message=asset_types.ImageAsset,
    )
    text_asset: asset_types.TextAsset = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="asset_data",
        message=asset_types.TextAsset,
    )
    lead_form_asset: asset_types.LeadFormAsset = proto.Field(
        proto.MESSAGE,
        number=9,
        oneof="asset_data",
        message=asset_types.LeadFormAsset,
    )
    book_on_google_asset: asset_types.BookOnGoogleAsset = proto.Field(
        proto.MESSAGE,
        number=10,
        oneof="asset_data",
        message=asset_types.BookOnGoogleAsset,
    )
    promotion_asset: asset_types.PromotionAsset = proto.Field(
        proto.MESSAGE,
        number=15,
        oneof="asset_data",
        message=asset_types.PromotionAsset,
    )
    callout_asset: asset_types.CalloutAsset = proto.Field(
        proto.MESSAGE,
        number=20,
        oneof="asset_data",
        message=asset_types.CalloutAsset,
    )
    structured_snippet_asset: asset_types.StructuredSnippetAsset = proto.Field(
        proto.MESSAGE,
        number=21,
        oneof="asset_data",
        message=asset_types.StructuredSnippetAsset,
    )
    sitelink_asset: asset_types.SitelinkAsset = proto.Field(
        proto.MESSAGE,
        number=22,
        oneof="asset_data",
        message=asset_types.SitelinkAsset,
    )
    page_feed_asset: asset_types.PageFeedAsset = proto.Field(
        proto.MESSAGE,
        number=23,
        oneof="asset_data",
        message=asset_types.PageFeedAsset,
    )
    dynamic_education_asset: asset_types.DynamicEducationAsset = proto.Field(
        proto.MESSAGE,
        number=24,
        oneof="asset_data",
        message=asset_types.DynamicEducationAsset,
    )
    mobile_app_asset: asset_types.MobileAppAsset = proto.Field(
        proto.MESSAGE,
        number=25,
        oneof="asset_data",
        message=asset_types.MobileAppAsset,
    )
    hotel_callout_asset: asset_types.HotelCalloutAsset = proto.Field(
        proto.MESSAGE,
        number=26,
        oneof="asset_data",
        message=asset_types.HotelCalloutAsset,
    )
    call_asset: asset_types.CallAsset = proto.Field(
        proto.MESSAGE,
        number=27,
        oneof="asset_data",
        message=asset_types.CallAsset,
    )
    price_asset: asset_types.PriceAsset = proto.Field(
        proto.MESSAGE,
        number=28,
        oneof="asset_data",
        message=asset_types.PriceAsset,
    )
    call_to_action_asset: asset_types.CallToActionAsset = proto.Field(
        proto.MESSAGE,
        number=29,
        oneof="asset_data",
        message=asset_types.CallToActionAsset,
    )
    dynamic_real_estate_asset: asset_types.DynamicRealEstateAsset = proto.Field(
        proto.MESSAGE,
        number=30,
        oneof="asset_data",
        message=asset_types.DynamicRealEstateAsset,
    )
    dynamic_custom_asset: asset_types.DynamicCustomAsset = proto.Field(
        proto.MESSAGE,
        number=31,
        oneof="asset_data",
        message=asset_types.DynamicCustomAsset,
    )
    dynamic_hotels_and_rentals_asset: (
        asset_types.DynamicHotelsAndRentalsAsset
    ) = proto.Field(
        proto.MESSAGE,
        number=32,
        oneof="asset_data",
        message=asset_types.DynamicHotelsAndRentalsAsset,
    )
    dynamic_flights_asset: asset_types.DynamicFlightsAsset = proto.Field(
        proto.MESSAGE,
        number=33,
        oneof="asset_data",
        message=asset_types.DynamicFlightsAsset,
    )
    demand_gen_carousel_card_asset: asset_types.DemandGenCarouselCardAsset = (
        proto.Field(
            proto.MESSAGE,
            number=50,
            oneof="asset_data",
            message=asset_types.DemandGenCarouselCardAsset,
        )
    )
    dynamic_travel_asset: asset_types.DynamicTravelAsset = proto.Field(
        proto.MESSAGE,
        number=35,
        oneof="asset_data",
        message=asset_types.DynamicTravelAsset,
    )
    dynamic_local_asset: asset_types.DynamicLocalAsset = proto.Field(
        proto.MESSAGE,
        number=36,
        oneof="asset_data",
        message=asset_types.DynamicLocalAsset,
    )
    dynamic_jobs_asset: asset_types.DynamicJobsAsset = proto.Field(
        proto.MESSAGE,
        number=37,
        oneof="asset_data",
        message=asset_types.DynamicJobsAsset,
    )
    location_asset: asset_types.LocationAsset = proto.Field(
        proto.MESSAGE,
        number=39,
        oneof="asset_data",
        message=asset_types.LocationAsset,
    )
    hotel_property_asset: asset_types.HotelPropertyAsset = proto.Field(
        proto.MESSAGE,
        number=41,
        oneof="asset_data",
        message=asset_types.HotelPropertyAsset,
    )
    business_message_asset: asset_types.BusinessMessageAsset = proto.Field(
        proto.MESSAGE,
        number=51,
        oneof="asset_data",
        message=asset_types.BusinessMessageAsset,
    )
    app_deep_link_asset: asset_types.AppDeepLinkAsset = proto.Field(
        proto.MESSAGE,
        number=52,
        oneof="asset_data",
        message=asset_types.AppDeepLinkAsset,
    )
    youtube_video_list_asset: asset_types.YouTubeVideoListAsset = proto.Field(
        proto.MESSAGE,
        number=53,
        oneof="asset_data",
        message=asset_types.YouTubeVideoListAsset,
    )


class AssetFieldTypePolicySummary(proto.Message):
    r"""Contains policy information for an asset under AssetFieldType
    context.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        asset_field_type (google.ads.googleads.v21.enums.types.AssetFieldTypeEnum.AssetFieldType):
            Output only. FieldType of this asset.

            This field is a member of `oneof`_ ``_asset_field_type``.
        asset_source (google.ads.googleads.v21.enums.types.AssetSourceEnum.AssetSource):
            Output only. Source of this asset.

            This field is a member of `oneof`_ ``_asset_source``.
        policy_summary_info (google.ads.googleads.v21.resources.types.AssetPolicySummary):
            Output only. Policy summary.

            This field is a member of `oneof`_ ``_policy_summary_info``.
    """

    asset_field_type: (
        gage_asset_field_type.AssetFieldTypeEnum.AssetFieldType
    ) = proto.Field(
        proto.ENUM,
        number=1,
        optional=True,
        enum=gage_asset_field_type.AssetFieldTypeEnum.AssetFieldType,
    )
    asset_source: gage_asset_source.AssetSourceEnum.AssetSource = proto.Field(
        proto.ENUM,
        number=2,
        optional=True,
        enum=gage_asset_source.AssetSourceEnum.AssetSource,
    )
    policy_summary_info: "AssetPolicySummary" = proto.Field(
        proto.MESSAGE,
        number=3,
        optional=True,
        message="AssetPolicySummary",
    )


class AssetPolicySummary(proto.Message):
    r"""Contains policy information for an asset.

    Attributes:
        policy_topic_entries (MutableSequence[google.ads.googleads.v21.common.types.PolicyTopicEntry]):
            Output only. The list of policy findings for
            this asset.
        review_status (google.ads.googleads.v21.enums.types.PolicyReviewStatusEnum.PolicyReviewStatus):
            Output only. Where in the review process this
            asset is.
        approval_status (google.ads.googleads.v21.enums.types.PolicyApprovalStatusEnum.PolicyApprovalStatus):
            Output only. The overall approval status of
            this asset, calculated based on the status of
            its individual policy topic entries.
    """

    policy_topic_entries: MutableSequence[policy.PolicyTopicEntry] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=policy.PolicyTopicEntry,
        )
    )
    review_status: (
        policy_review_status.PolicyReviewStatusEnum.PolicyReviewStatus
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=policy_review_status.PolicyReviewStatusEnum.PolicyReviewStatus,
    )
    approval_status: (
        policy_approval_status.PolicyApprovalStatusEnum.PolicyApprovalStatus
    ) = proto.Field(
        proto.ENUM,
        number=3,
        enum=policy_approval_status.PolicyApprovalStatusEnum.PolicyApprovalStatus,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.errors",
    marshal="google.ads.googleads.v21",
    manifest={
        "AdGroupErrorEnum",
    },
)


class AdGroupErrorEnum(proto.Message):
    r"""Container for enum describing possible ad group errors."""

    class AdGroupError(proto.Enum):
        r"""Enum describing possible ad group errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            DUPLICATE_ADGROUP_NAME (2):
                AdGroup with the same name already exists for
                the campaign.
            INVALID_ADGROUP_NAME (3):
                AdGroup name is not valid.
            ADVERTISER_NOT_ON_CONTENT_NETWORK (5):
                Advertiser is not allowed to target sites or
                set site bids that are not on the Google Search
                Network.
            BID_TOO_BIG (6):
                Bid amount is too big.
            BID_TYPE_AND_BIDDING_STRATEGY_MISMATCH (7):
                AdGroup bid does not match the campaign's
                bidding strategy.
            MISSING_ADGROUP_NAME (8):
                AdGroup name is required for Add.
            ADGROUP_LABEL_DOES_NOT_EXIST (9):
                No link found between the ad group and the
                label.
            ADGROUP_LABEL_ALREADY_EXISTS (10):
                The label has already been attached to the ad
                group.
            INVALID_CONTENT_BID_CRITERION_TYPE_GROUP (11):
                The CriterionTypeGroup is not supported for
                the content bid dimension.
            AD_GROUP_TYPE_NOT_VALID_FOR_ADVERTISING_CHANNEL_TYPE (12):
                The ad group type is not compatible with the
                campaign channel type.
            ADGROUP_TYPE_NOT_SUPPORTED_FOR_CAMPAIGN_SALES_COUNTRY (13):
                The ad group type is not supported in the
                country of sale of the campaign.
            CANNOT_ADD_ADGROUP_OF_TYPE_DSA_TO_CAMPAIGN_WITHOUT_DSA_SETTING (14):
                Ad groups of AdGroupType.SEARCH_DYNAMIC_ADS can only be
                added to campaigns that have DynamicSearchAdsSetting
                attached.
            PROMOTED_HOTEL_AD_GROUPS_NOT_AVAILABLE_FOR_CUSTOMER (15):
                Promoted hotels ad groups are only available
                to customers on the allow-list.
            INVALID_EXCLUDED_PARENT_ASSET_FIELD_TYPE (16):
                The field type cannot be excluded because an
                active ad group-asset link of this type exists.
            INVALID_EXCLUDED_PARENT_ASSET_SET_TYPE (17):
                The asset set type is invalid for setting the
                excluded_parent_asset_set_types field.
            CANNOT_ADD_AD_GROUP_FOR_CAMPAIGN_TYPE (18):
                Cannot add ad groups for the campaign type.
            INVALID_STATUS (19):
                Invalid status for the ad group.
            INVALID_STEP_ID_FOR_VIDEO_ADS_SEQUENCING (20):
                For video ads sequencing, AdGroup ``step_id`` has to use a
                ``step_id`` defined in
                ``campaign.video_campaign_settings.video_ad_sequence``.
            INVALID_AD_GROUP_TYPE_FOR_VIDEO_ADS_SEQUENCING (21):
                For video ads sequencing, AdGroup type has to use a type
                defined in
                ``campaign.video_campaign_settings.video_ad_sequence``.
            DUPLICATE_STEP_ID (22):
                Only one AdGroup is allowed for each step ID
                in video ads sequencing.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        DUPLICATE_ADGROUP_NAME = 2
        INVALID_ADGROUP_NAME = 3
        ADVERTISER_NOT_ON_CONTENT_NETWORK = 5
        BID_TOO_BIG = 6
        BID_TYPE_AND_BIDDING_STRATEGY_MISMATCH = 7
        MISSING_ADGROUP_NAME = 8
        ADGROUP_LABEL_DOES_NOT_EXIST = 9
        ADGROUP_LABEL_ALREADY_EXISTS = 10
        INVALID_CONTENT_BID_CRITERION_TYPE_GROUP = 11
        AD_GROUP_TYPE_NOT_VALID_FOR_ADVERTISING_CHANNEL_TYPE = 12
        ADGROUP_TYPE_NOT_SUPPORTED_FOR_CAMPAIGN_SALES_COUNTRY = 13
        CANNOT_ADD_ADGROUP_OF_TYPE_DSA_TO_CAMPAIGN_WITHOUT_DSA_SETTING = 14
        PROMOTED_HOTEL_AD_GROUPS_NOT_AVAILABLE_FOR_CUSTOMER = 15
        INVALID_EXCLUDED_PARENT_ASSET_FIELD_TYPE = 16
        INVALID_EXCLUDED_PARENT_ASSET_SET_TYPE = 17
        CANNOT_ADD_AD_GROUP_FOR_CAMPAIGN_TYPE = 18
        INVALID_STATUS = 19
        INVALID_STEP_ID_FOR_VIDEO_ADS_SEQUENCING = 20
        INVALID_AD_GROUP_TYPE_FOR_VIDEO_ADS_SEQUENCING = 21
        DUPLICATE_STEP_ID = 22


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.errors",
    marshal="google.ads.googleads.v22",
    manifest={
        "OfflineUserDataJobErrorEnum",
    },
)


class OfflineUserDataJobErrorEnum(proto.Message):
    r"""Container for enum describing possible offline user data job
    errors.

    """

    class OfflineUserDataJobError(proto.Enum):
        r"""Enum describing possible request errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            INVALID_USER_LIST_ID (3):
                The user list ID provided for the job is
                invalid.
            INVALID_USER_LIST_TYPE (4):
                Type of the user list is not applicable for
                the job.
            NOT_ON_ALLOWLIST_FOR_USER_ID (33):
                Customer is not allowisted for using user ID
                in upload data.
            INCOMPATIBLE_UPLOAD_KEY_TYPE (6):
                Upload data is not compatible with the upload
                key type of the associated user list.
            MISSING_USER_IDENTIFIER (7):
                The user identifier is missing valid data.
            INVALID_MOBILE_ID_FORMAT (8):
                The mobile ID is malformed.
            TOO_MANY_USER_IDENTIFIERS (9):
                Maximum number of user identifiers allowed
                per request is 100,000 and per operation is 20.
            NOT_ON_ALLOWLIST_FOR_STORE_SALES_DIRECT (31):
                Customer is not on the allow-list for store
                sales direct data.
            NOT_ON_ALLOWLIST_FOR_UNIFIED_STORE_SALES (32):
                Customer is not on the allow-list for unified
                store sales data.
            INVALID_PARTNER_ID (11):
                The partner ID in store sales direct metadata
                is invalid.
            INVALID_ENCODING (12):
                The data in user identifier should not be
                encoded.
            INVALID_COUNTRY_CODE (13):
                The country code is invalid.
            INCOMPATIBLE_USER_IDENTIFIER (14):
                Incompatible user identifier when using third_party_user_id
                for store sales direct first party data or not using
                third_party_user_id for store sales third party data.
            FUTURE_TRANSACTION_TIME (15):
                A transaction time in the future is not
                allowed.
            INVALID_CONVERSION_ACTION (16):
                The conversion_action specified in transaction_attributes is
                used to report conversions to a conversion action configured
                in Google Ads. This error indicates there is no such
                conversion action in the account.
            MOBILE_ID_NOT_SUPPORTED (17):
                Mobile ID is not supported for store sales
                direct data.
            INVALID_OPERATION_ORDER (18):
                When a remove-all operation is provided, it
                has to be the first operation of the operation
                list.
            CONFLICTING_OPERATION (19):
                Mixing creation and removal of offline data
                in the same job is not allowed.
            EXTERNAL_UPDATE_ID_ALREADY_EXISTS (21):
                The external update ID already exists.
            JOB_ALREADY_STARTED (22):
                Once the upload job is started, new
                operations cannot be added.
            REMOVE_NOT_SUPPORTED (23):
                Remove operation is not allowed for store
                sales direct updates.
            REMOVE_ALL_NOT_SUPPORTED (24):
                Remove-all is not supported for certain
                offline user data job types.
            INVALID_SHA256_FORMAT (25):
                The SHA256 encoded value is malformed.
            CUSTOM_KEY_DISABLED (26):
                The custom key specified is not enabled for
                the unified store sales upload.
            CUSTOM_KEY_NOT_PREDEFINED (27):
                The custom key specified is not predefined
                through the Google Ads UI.
            CUSTOM_KEY_NOT_SET (29):
                The custom key specified is not set in the
                upload.
            CUSTOMER_NOT_ACCEPTED_CUSTOMER_DATA_TERMS (30):
                The customer has not accepted the customer
                data terms in the conversion settings page.
            ATTRIBUTES_NOT_APPLICABLE_FOR_CUSTOMER_MATCH_USER_LIST (34):
                User attributes cannot be uploaded into a
                user list.
            LIFETIME_VALUE_BUCKET_NOT_IN_RANGE (35):
                Lifetime bucket value must be a number from 0
                to 10; 0 is only accepted for remove operations
            INCOMPATIBLE_USER_IDENTIFIER_FOR_ATTRIBUTES (36):
                Identifiers not supported for Customer Match
                attributes. User attributes can only be provided
                with contact info (email, phone, address) user
                identifiers.
            FUTURE_TIME_NOT_ALLOWED (37):
                A time in the future is not allowed.
            LAST_PURCHASE_TIME_LESS_THAN_ACQUISITION_TIME (38):
                Last purchase date time cannot be less than
                acquisition date time.
            CUSTOMER_IDENTIFIER_NOT_ALLOWED (39):
                Only emails are accepted as user identifiers
                for shopping loyalty match. {--
                api.dev/not-precedent: The identifier is not
                limited to ids, but also include other user info
                eg. phone numbers.}
            INVALID_ITEM_ID (40):
                Provided item ID is invalid.
            FIRST_PURCHASE_TIME_GREATER_THAN_LAST_PURCHASE_TIME (42):
                First purchase date time cannot be greater
                than the last purchase date time.
            INVALID_LIFECYCLE_STAGE (43):
                Provided lifecycle stage is invalid.
            INVALID_EVENT_VALUE (44):
                The event value of the Customer Match user
                attribute is invalid.
            EVENT_ATTRIBUTE_ALL_FIELDS_ARE_REQUIRED (45):
                All the fields are not present in the
                EventAttribute of the Customer Match.
            OPERATION_LEVEL_CONSENT_PROVIDED (48):
                Consent was provided at the operation level
                for an OfflineUserDataJobType that expects it at
                the job level. The provided operation-level
                consent will be ignored.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        INVALID_USER_LIST_ID = 3
        INVALID_USER_LIST_TYPE = 4
        NOT_ON_ALLOWLIST_FOR_USER_ID = 33
        INCOMPATIBLE_UPLOAD_KEY_TYPE = 6
        MISSING_USER_IDENTIFIER = 7
        INVALID_MOBILE_ID_FORMAT = 8
        TOO_MANY_USER_IDENTIFIERS = 9
        NOT_ON_ALLOWLIST_FOR_STORE_SALES_DIRECT = 31
        NOT_ON_ALLOWLIST_FOR_UNIFIED_STORE_SALES = 32
        INVALID_PARTNER_ID = 11
        INVALID_ENCODING = 12
        INVALID_COUNTRY_CODE = 13
        INCOMPATIBLE_USER_IDENTIFIER = 14
        FUTURE_TRANSACTION_TIME = 15
        INVALID_CONVERSION_ACTION = 16
        MOBILE_ID_NOT_SUPPORTED = 17
        INVALID_OPERATION_ORDER = 18
        CONFLICTING_OPERATION = 19
        EXTERNAL_UPDATE_ID_ALREADY_EXISTS = 21
        JOB_ALREADY_STARTED = 22
        REMOVE_NOT_SUPPORTED = 23
        REMOVE_ALL_NOT_SUPPORTED = 24
        INVALID_SHA256_FORMAT = 25
        CUSTOM_KEY_DISABLED = 26
        CUSTOM_KEY_NOT_PREDEFINED = 27
        CUSTOM_KEY_NOT_SET = 29
        CUSTOMER_NOT_ACCEPTED_CUSTOMER_DATA_TERMS = 30
        ATTRIBUTES_NOT_APPLICABLE_FOR_CUSTOMER_MATCH_USER_LIST = 34
        LIFETIME_VALUE_BUCKET_NOT_IN_RANGE = 35
        INCOMPATIBLE_USER_IDENTIFIER_FOR_ATTRIBUTES = 36
        FUTURE_TIME_NOT_ALLOWED = 37
        LAST_PURCHASE_TIME_LESS_THAN_ACQUISITION_TIME = 38
        CUSTOMER_IDENTIFIER_NOT_ALLOWED = 39
        INVALID_ITEM_ID = 40
        FIRST_PURCHASE_TIME_GREATER_THAN_LAST_PURCHASE_TIME = 42
        INVALID_LIFECYCLE_STAGE = 43
        INVALID_EVENT_VALUE = 44
        EVENT_ATTRIBUTE_ALL_FIELDS_ARE_REQUIRED = 45
        OPERATION_LEVEL_CONSENT_PROVIDED = 48


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2018 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example illustrates adding a conversion action."""


import argparse
import sys
import uuid

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.resources.types.conversion_action import (
    ConversionAction,
)
from google.ads.googleads.v22.services.services.conversion_action_service import (
    ConversionActionServiceClient,
)
from google.ads.googleads.v22.services.types.conversion_action_service import (
    ConversionActionOperation,
    MutateConversionActionsResponse,
)


# [START add_conversion_action]
def main(client: GoogleAdsClient, customer_id: str) -> None:
    conversion_action_service: ConversionActionServiceClient = (
        client.get_service("ConversionActionService")
    )

    # Create the operation.
    conversion_action_operation: ConversionActionOperation = client.get_type(
        "ConversionActionOperation"
    )

    # Create conversion action.
    conversion_action: ConversionAction = conversion_action_operation.create

    # Note that conversion action names must be unique. If a conversion action
    # already exists with the specified conversion_action_name, the create
    # operation will fail with a ConversionActionError.DUPLICATE_NAME error.
    conversion_action.name = f"Earth to Mars Cruises Conversion {uuid.uuid4()}"
    conversion_action.type_ = (
        client.enums.ConversionActionTypeEnum.UPLOAD_CLICKS
    )
    conversion_action.category = (
        client.enums.ConversionActionCategoryEnum.DEFAULT
    )
    conversion_action.status = client.enums.ConversionActionStatusEnum.ENABLED
    conversion_action.view_through_lookback_window_days = 15

    # Create a value settings object.
    value_settings: ConversionAction.ValueSettings = (
        conversion_action.value_settings
    )
    value_settings.default_value = 15.0
    value_settings.always_use_default_value = True

    # Add the conversion action.
    conversion_action_response: MutateConversionActionsResponse = (
        conversion_action_service.mutate_conversion_actions(
            customer_id=customer_id,
            operations=[conversion_action_operation],
        )
    )

    print(
        "Created conversion action "
        f'"{conversion_action_response.results[0].resource_name}".'
    )
    # [END add_conversion_action]


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description="Adds a conversion action for specified customer."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(googleads_client, args.customer_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)
